@@ -4,7 +4,7 @@ int quicktime_atom_read_header(quicktime_t *file, quicktime_atom_t *atom);
 int quicktime_atom_write_header64(quicktime_t *file, quicktime_atom_t *atom, char *text);
 int quicktime_atom_write_header(quicktime_t *file, quicktime_atom_t *atom, char *text);
 void quicktime_atom_write_footer(quicktime_t *file, quicktime_atom_t *atom);
-int quicktime_atom_is(quicktime_atom_t *atom, char *type);
+int quicktime_atom_is(quicktime_atom_t *atom, unsigned char *type);
 int quicktime_atom_skip(quicktime_t *file, quicktime_atom_t *atom);
 
 /* avi_hdrl.c */
@@ -90,7 +90,7 @@ void quicktime_init_odml(quicktime_t *file, quicktime_hdrl_t *hdrl);
 /* avi_riff.c */
 void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom);
 void quicktime_delete_riff(quicktime_t *file, quicktime_riff_t *riff);
-void quicktime_import_avi(quicktime_t *file);
+int quicktime_import_avi(quicktime_t *file);
 void quicktime_init_riff(quicktime_t *file);
 void quicktime_finalize_riff(quicktime_t *file, quicktime_riff_t *riff);
 quicktime_riff_t* quicktime_new_riff(quicktime_t *file);
@@ -108,8 +108,6 @@ void quicktime_init_strl(quicktime_t *file,
                          quicktime_trak_t *trak,
                          quicktime_strl_t *strl);
 
-
-
 /* ctab.c */
 
 int quicktime_ctab_init(quicktime_ctab_t *ctab);
@@ -164,10 +162,27 @@ void quicktime_elst_dump(quicktime_elst_t *elst);
 void quicktime_read_elst(quicktime_t *file, quicktime_elst_t *elst);
 void quicktime_write_elst(quicktime_t *file, quicktime_elst_t *elst, long duration);
 
+/* gmhd.c */
+
+void quicktime_gmhd_init(quicktime_gmhd_t *gmhd);
+void quicktime_gmhd_delete(quicktime_gmhd_t *gmhd);
+void quicktime_gmhd_dump(quicktime_gmhd_t *gmhd);
+void quicktime_read_gmhd(quicktime_t *file, quicktime_gmhd_t *gmhd, quicktime_atom_t *parent_atom);
+void quicktime_write_gmhd(quicktime_t *file, quicktime_gmhd_t *gmhd);
+
+/* gmin.c */
+
+void quicktime_gmin_init(quicktime_gmin_t *gmin);
+void quicktime_gmin_delete(quicktime_gmin_t *gmin);
+void quicktime_gmin_dump(quicktime_gmin_t *gmin);
+void quicktime_read_gmin(quicktime_t *file, quicktime_gmin_t *gmin);
+void quicktime_write_gmin(quicktime_t *file, quicktime_gmin_t *gmin);
+
 /* hdlr.c */
 
 void quicktime_hdlr_init(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_init_video(quicktime_hdlr_t *hdlr);
+void quicktime_hdlr_init_panorama(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_init_audio(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_init_data(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_delete(quicktime_hdlr_t *hdlr);
@@ -202,11 +217,12 @@ void quicktime_mdhd_init_audio(quicktime_mdhd_t *mdhd,
                                int sample_rate);
 
 void quicktime_mdhd_init_video(quicktime_t *file, 
-                               quicktime_mdhd_t *mdhd, 
-                               int frame_w,
-                               int frame_h, 
-                               double frame_rate);
-
+                               quicktime_mdhd_t *mdhd,
+                               int timescale);
+			       
+void quicktime_mdhd_init_panorama(quicktime_t *file, 
+                               quicktime_mdhd_t *mdhd,
+                               int timescale);
 /* mdia.c */
 
 void quicktime_mdia_init(quicktime_mdia_t *mdia);
@@ -226,9 +242,17 @@ void quicktime_mdia_init_video(quicktime_t *file,
                                quicktime_mdia_t *mdia,
                                int frame_w,
                                int frame_h, 
-                               double frame_rate,
+                               int frame_duration,
+                               int timescale,
                                char *compressor);
 
+void quicktime_mdia_init_panorama(quicktime_t *file,
+				  quicktime_mdia_t *mdia,
+				  int width,
+				  int height,
+				  int frame_duration,
+				  int timescale);
+			       
 /* minf.c */
 
 void quicktime_minf_init(quicktime_minf_t *minf);
@@ -242,8 +266,8 @@ void quicktime_minf_init_video(quicktime_t *file,
                                quicktime_minf_t *minf, 
                                int frame_w,
                                int frame_h, 
+                               int frame_duration,
                                int time_scale, 
-                               double frame_rate,
                                char *compressor);
 
 void quicktime_minf_init_audio(quicktime_t *file, 
@@ -253,6 +277,12 @@ void quicktime_minf_init_audio(quicktime_t *file,
                                int bits, 
                                char *compressor);
 
+void quicktime_minf_init_panorama(quicktime_t *file,
+				  quicktime_minf_t *minf,
+				  int width,
+				  int height,
+				  int frame_duration);
+
 /* moov.c */
 
 int quicktime_moov_init(quicktime_moov_t *moov);
@@ -269,9 +299,57 @@ int quicktime_mvhd_init(quicktime_mvhd_t *mvhd);
 int quicktime_mvhd_delete(quicktime_mvhd_t *mvhd);
 void quicktime_mvhd_dump(quicktime_mvhd_t *mvhd);
 void quicktime_read_mvhd(quicktime_t *file, quicktime_mvhd_t *mvhd, quicktime_atom_t *parent_atom);
-void quicktime_mhvd_init_video(quicktime_t *file, quicktime_mvhd_t *mvhd, double frame_rate);
+void quicktime_mhvd_init_video(quicktime_t *file, quicktime_mvhd_t *mvhd, int time_scale);
 void quicktime_write_mvhd(quicktime_t *file, quicktime_mvhd_t *mvhd);
 
+/* navg */
+
+int quicktime_navg_init(quicktime_navg_t *navg);
+int quicktime_navg_delete(quicktime_navg_t *navg);
+void quicktime_navg_dump(quicktime_navg_t *navg);
+int quicktime_read_navg(quicktime_t *file, quicktime_navg_t *navg, quicktime_atom_t *navg_atom);
+void quicktime_write_navg(quicktime_t *file, quicktime_navg_t *navg);
+
+/* colr.c */
+void quicktime_colr_init(quicktime_colr_t *colr);
+void quicktime_colr_delete(quicktime_colr_t *colr);
+void quicktime_colr_dump(quicktime_colr_t *colr);
+void quicktime_read_colr(quicktime_t *file, quicktime_colr_t *colr);
+void quicktime_write_colr(quicktime_t *file, quicktime_colr_t *colr);
+int  lqt_set_colr(quicktime_t *file, int track, quicktime_colr_t *colr);
+int  lqt_get_colr(quicktime_t *file, int track, quicktime_colr_t *colr);
+
+/* clap.c */
+void quicktime_clap_init(quicktime_clap_t *clap);
+void quicktime_clap_delete(quicktime_clap_t *clap);
+void quicktime_clap_dump(quicktime_clap_t *clap);
+void quicktime_read_clap(quicktime_t *file, quicktime_clap_t *clap);
+void quicktime_write_clap(quicktime_t *file, quicktime_clap_t *clap);
+int  lqt_set_clap(quicktime_t *file, int track, quicktime_clap_t *clap);
+int  lqt_get_clap(quicktime_t *file, int track, quicktime_clap_t *clap);
+
+/* pano */
+
+int quicktime_pano_init(quicktime_pano_t *pano);  
+int quicktime_pano_delete(quicktime_pano_t *pano);
+void quicktime_pano_dump(quicktime_pano_t *pano);
+int quicktime_read_pano(quicktime_t *file, quicktime_pano_t *navg, quicktime_atom_t *pano_atom);
+void quicktime_write_pano(quicktime_t *file, quicktime_pano_t *pano);
+
+/* pasp.c */
+void quicktime_pasp_init(quicktime_pasp_t *pasp);
+void quicktime_pasp_delete(quicktime_pasp_t *pasp);
+void quicktime_pasp_dump(quicktime_pasp_t *pasp);
+void quicktime_read_pasp(quicktime_t *file, quicktime_pasp_t *pasp);
+void quicktime_write_pasp(quicktime_t *file, quicktime_pasp_t *pasp);
+int  lqt_set_pasp(quicktime_t *file, int track, quicktime_pasp_t *pasp);
+int  lqt_get_pasp(quicktime_t *file, int track, quicktime_pasp_t *pasp);
+
+
+/* pHdr */
+
+int quicktime_pHdr_init(quicktime_pHdr_t *pHdr);
+
 /* smhd.c */
 
 void quicktime_smhd_init(quicktime_smhd_t *smhd);
@@ -295,8 +373,8 @@ void quicktime_stbl_init_video(quicktime_t *file,
                                quicktime_stbl_t *stbl, 
                                int frame_w,
                                int frame_h, 
-                               int time_scale, 
-                               double frame_rate,
+                               int frame_duration,
+                               int time_scale,
                                char *compressor);
 
 void quicktime_stbl_init_audio(quicktime_t *file, 
@@ -305,7 +383,14 @@ void quicktime_stbl_init_audio(quicktime_t *file,
                                int sample_rate, 
                                int bits, 
                                char *compressor);
+			       
+void quicktime_stbl_init_panorama(quicktime_t *file,
+				  quicktime_stbl_t *stbl,
+				  int width,
+				  int height,
+				  int frame_duration);
 
+			     
 /* stco.c */
 
 void quicktime_stco_init(quicktime_stco_t *stco);
@@ -342,7 +427,6 @@ void quicktime_stsd_init_video(quicktime_t *file,
                                quicktime_stsd_t *stsd, 
                                int frame_w,
                                int frame_h, 
-                               double frame_rate,
                                char *compression);
 
 void quicktime_stsd_init_audio(quicktime_t *file, 
@@ -352,6 +436,11 @@ void quicktime_stsd_init_audio(quicktime_t *file,
                                int bits, 
                                char *compressor);
 
+void quicktime_stsd_init_panorama(quicktime_t *file,
+				  quicktime_stsd_t *stsd,
+				  int width,
+				  int height);
+			       
 /* stsdtable.c */
 
 void quicktime_mjqt_init(quicktime_mjqt_t *mjqt);
@@ -408,12 +497,17 @@ void quicktime_update_stsz(quicktime_stsz_t *stsz,
 
 void quicktime_stts_init(quicktime_stts_t *stts);
 void quicktime_stts_init_table(quicktime_stts_t *stts);
-void quicktime_stts_init_video(quicktime_t *file, quicktime_stts_t *stts, int time_scale, double frame_rate);
+void quicktime_stts_init_video(quicktime_t *file, quicktime_stts_t *stts, int frame_duration);
 void quicktime_stts_init_audio(quicktime_t *file, quicktime_stts_t *stts, int sample_rate);
 void quicktime_stts_delete(quicktime_stts_t *stts);
 void quicktime_stts_dump(quicktime_stts_t *stts);
 void quicktime_read_stts(quicktime_t *file, quicktime_stts_t *stts);
 void quicktime_write_stts(quicktime_t *file, quicktime_stts_t *stts);
+void quicktime_update_stts(quicktime_stts_t *stts, long sample, long duration);
+/* The following function kicks out redundant entries */
+void quicktime_compress_stts(quicktime_stts_t *stts);
+
+void quicktime_stts_init_panorama(quicktime_t *file, quicktime_stts_t *stts, int frame_duration);
 
 
 int64_t quicktime_time_to_sample(quicktime_stts_t *stts, int64_t * time,
@@ -440,7 +534,8 @@ int quicktime_trak_init_video(quicktime_t *file,
                               quicktime_trak_t *trak, 
                               int frame_w, 
                               int frame_h, 
-                              double frame_rate,
+                              int frame_duration,
+                              int time_scale,
                               char *compressor);
 
 int quicktime_trak_init_audio(quicktime_t *file, 
@@ -449,6 +544,13 @@ int quicktime_trak_init_audio(quicktime_t *file,
                               int sample_rate, 
                               int bits, 
                               char *compressor);
+			      
+int quicktime_trak_init_panorama(quicktime_t *file,
+				 quicktime_trak_t *trak,
+				 int width,
+				 int height,
+				 int timescale,
+				 int frame_duration);
 
 int quicktime_trak_delete(quicktime_trak_t *trak);
 int quicktime_trak_dump(quicktime_trak_t *trak);
@@ -513,13 +615,16 @@ void quicktime_vmhd_init_video(quicktime_t *file,
                                quicktime_vmhd_t *vmhd, 
                                int frame_w,
                                int frame_h, 
-                               double frame_rate);
+                               int frame_duration,
+                               int timescale);
 
 void quicktime_vmhd_delete(quicktime_vmhd_t *vmhd);
 void quicktime_vmhd_dump(quicktime_vmhd_t *vmhd);
 void quicktime_read_vmhd(quicktime_t *file, quicktime_vmhd_t *vmhd);
 void quicktime_write_vmhd(quicktime_t *file, quicktime_vmhd_t *vmhd);
 
+
+
 /* util.c */
 
 /* Disk I/O */
@@ -584,6 +689,7 @@ int quicktime_delete_video_map(quicktime_video_map_t *vtrack);
 int quicktime_update_positions(quicktime_t *file);
 
 void quicktime_init_maps(quicktime_t * file);
+void lqt_update_frame_position(quicktime_video_map_t * track);
 
 /*
  *  Convenience function: Returns an array of chunk sizes
@@ -596,6 +702,7 @@ void quicktime_init_maps(quicktime_t * file);
 
 int64_t * lqt_get_chunk_sizes(quicktime_t * file, quicktime_trak_t *trak);
 
+
 /* lqt_codecs.c */
 
 int quicktime_delete_vcodec(quicktime_video_map_t *vtrack);
@@ -603,7 +710,6 @@ int quicktime_delete_acodec(quicktime_audio_map_t *vtrack);
 int quicktime_codecs_flush(quicktime_t *file);
 void quicktime_id_to_codec(char *result, int id);
 int quicktime_codec_to_id(char *codec);
-int64_t quicktime_samples_to_bytes(quicktime_trak_t *track, long samples);
 
 /* workarounds.c */
 
