@@ -38,9 +38,22 @@
 int ffmpeg_num_audio_codecs = -1;
 int ffmpeg_num_video_codecs = -1;
 
+
+static int encoding_colormodels_ffmpeg[] = {
+	BC_YUV420P, 
+	LQT_COLORMODEL_NONE
+};
+
+static int encoding_colormodels_dvc[] = {
+	BC_YUV411P, 
+	LQT_COLORMODEL_NONE
+};
+
+
+
 #define ENCODE_PARAM_AUDIO \
 	{\
-	  "bit_rate",\
+	  "bit_rate_audio",\
           "Bit rate (kbps)",\
 	  LQT_PARAMETER_INT,\
           { 128 },\
@@ -90,7 +103,7 @@ int ffmpeg_num_video_codecs = -1;
         LQT_PARAMETER_SECTION, \
         }, \
         {\
-	  "bit_rate",\
+	  "bit_rate_video",\
           "Bit rate (kbps)",\
 	  LQT_PARAMETER_INT,\
           { 800 },\
@@ -182,7 +195,7 @@ int ffmpeg_num_video_codecs = -1;
           31 \
         }, \
         {\
-          "qdiff", \
+          "max_qdiff", \
           "max difference between the quantiser scale (VBR)", \
           LQT_PARAMETER_INT, \
           { 3 }, \
@@ -352,12 +365,12 @@ static lqt_parameter_info_static_t decode_parameters_mpeg4[] = {
   DECODE_PARAM_VIDEO,
   { /* End of parameters */ }
 };
-
+#if 0
 static lqt_parameter_info_static_t decode_parameters_rle[] = {
   DECODE_PARAM_VIDEO,
   { /* End of parameters */ }
 };
-
+#endif
 
 static lqt_parameter_info_static_t decode_parameters_audio[] = {
   //  DECODE_PARAM_AUDIO,
@@ -375,22 +388,28 @@ struct CODECIDMAP {
 	char *name;
 	char *fourccs[MAX_FOURCCS];
         int   wav_ids[MAX_WAV_IDS];
-};
+        int * encoding_colormodels;
+/*
+ *  We explicitely allow, if encoding is allowed.
+ *  This prevents the spreading of broken files
+ */
+        int   do_encode; 
+  };
 
 struct CODECIDMAP codecidmap_v[] = {
 /* Tables from mplayers config... */
 /* Video */
-	{
+        {
           id: CODEC_ID_MPEG1VIDEO,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpegvideo,
+          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "mpg1",
 	  name: "Mpeg 1 Video",
 	  fourccs: {"mpg1", "MPG1", "pim1", "PIM1", (char *)0} },
-	{
+        {
           id: CODEC_ID_MPEG4,
 	  index: -1,
           encoder: NULL,
@@ -399,15 +418,17 @@ struct CODECIDMAP codecidmap_v[] = {
           decode_parameters: decode_parameters_mpeg4,
 	  short_name: "mpg4",
 	  name: "Mpeg 4 Video (DivX)",
-	  fourccs: {"DIVX", "divx", "DIV1", "div1", "MP4S", "mp4s", "M4S2",
-                    "m4s2", "xvid", "XVID", "XviD", "DX50", "dx50", "mp4v",
-                    "MP4V", (char *)0} },
+	  fourccs: {"mp4v", "divx", "DIV1", "div1", "MP4S", "mp4s", "M4S2",
+                    "m4s2", "xvid", "XVID", "XviD", "DX50", "dx50", "DIVX",
+                    "MP4V", (char *)0 },
+          encoding_colormodels: encoding_colormodels_ffmpeg,
+          do_encode: 1 },
 	{
           id: CODEC_ID_MSMPEG4V1,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpeg4,
+          //          encode_parameters: encode_parameters_mpeg4,
           decode_parameters: decode_parameters_mpeg4,
 	  short_name: "msmpeg4v1",
 	  name: "MSMpeg 4v1",
@@ -417,7 +438,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpeg4,
+          //          encode_parameters: encode_parameters_mpeg4,
           decode_parameters: decode_parameters_mpeg4,
 	  short_name: "msmpeg4v2",
 	  name: "MSMpeg 4v2",
@@ -433,7 +454,9 @@ struct CODECIDMAP codecidmap_v[] = {
 	  name: "MSMpeg 4v3",
 	  fourccs: {"DIV3", "mpg3", "MP43", "mp43", "DIV5", "div5", "DIV6",
                     "MPG3", "div6", "div3", "DIV4", "div4", "AP41", "ap41",
-                    (char *)0}
+                    (char *)0},
+          encoding_colormodels: encoding_colormodels_ffmpeg,
+          do_encode: 1,
         },
 #if 0
 	{
@@ -456,7 +479,9 @@ struct CODECIDMAP codecidmap_v[] = {
           decode_parameters: decode_parameters_video,
 	  short_name: "h263",
 	  name: "H263",
-	  fourccs: {"H263", "h263", (char *)0} },
+	  fourccs: {"H263", "h263", "U263", "u263", (char *)0},
+          encoding_colormodels: encoding_colormodels_ffmpeg,
+          do_encode: 1, },
 	{
           id: CODEC_ID_H263P,
 	  index: -1,
@@ -466,17 +491,21 @@ struct CODECIDMAP codecidmap_v[] = {
           decode_parameters: decode_parameters_video,
 	  short_name: "h263p",
 	  name: "H263+",
-	  fourccs: {"U263", "u263", (char *)0} },
+	  fourccs: {"U263", "u263", (char *)0},
+          encoding_colormodels: encoding_colormodels_ffmpeg,
+          do_encode: 1, },
 	{
           id: CODEC_ID_H263I,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpegvideo,
+          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "i263",
 	  name: "I263",
-	  fourccs: {"I263", "i263", "viv1", "VIV1", (char *)0} },
+          encoding_colormodels: encoding_colormodels_ffmpeg,
+	  fourccs: {"I263", "i263", "viv1", "VIV1", (char *)0},
+        },
 #if 0
 	{
           id: CODEC_ID_RV10,
@@ -494,7 +523,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpegvideo,
+          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "svq1",
 	  name: "Sorenson Video 1",
@@ -504,7 +533,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpegvideo,
+          //          encode_parameters: encode_parameters_mpegvideo,
           decode_parameters: decode_parameters_video,
 	  short_name: "svq3",
 	  name: "Sorenson Video 3",
@@ -518,13 +547,15 @@ struct CODECIDMAP codecidmap_v[] = {
           decode_parameters: decode_parameters_video,
 	  short_name: "mjpg",
 	  name: "MJPEG",
-	  fourccs: {"MJPG", "mjpg", "JPEG", "jpeg", "dmb1", (char *)0} },
+	  fourccs: {"MJPG", "mjpg", "JPEG", "jpeg", "dmb1", "AVDJ", (char *)0},
+          encoding_colormodels: encoding_colormodels_ffmpeg,
+          do_encode: 1, },
 	{
           id: CODEC_ID_8BPS,
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "8BPS",
 	  name: "Quicktime Planar RGB (8BPS)",
@@ -534,7 +565,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "indeo",
 	  name: "Intel Indeo 3",
@@ -544,7 +575,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "rpza",
 	  name: "Apple Video",
@@ -554,7 +585,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "smc",
 	  name: "Apple Graphics",
@@ -564,7 +595,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "cinepak",
 	  name: "Cinepak",
@@ -574,7 +605,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "cyuv",
 	  name: "Creative YUV",
@@ -584,16 +615,61 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_video,
+          //          encode_parameters: encode_parameters_video,
           decode_parameters: decode_parameters_video,
 	  short_name: "rle",
 	  name: "RLE",
 	  fourccs: {"rle ", (char *)0} },
+        {
+          id: CODEC_ID_MSRLE,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          //          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "wrle",
+	  name: "Microsoft RLE",
+	  fourccs: {"WRLE", (char *)0} },
+        {
+          id: CODEC_ID_DVVIDEO,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "dv_ntsc",
+	  name: "DV (NTSC)",
+	  fourccs: {"dvc ", (char *)0 },
+          encoding_colormodels: encoding_colormodels_dvc,
+          do_encode: 1 },
+        {
+          id: CODEC_ID_DVVIDEO,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "dv_pal",
+	  name: "DV (PAL)",
+	  fourccs: {"dvcp", "dvpp", (char *)0 },
+          encoding_colormodels: encoding_colormodels_ffmpeg,
+          do_encode: 1 },
 };
 
 struct CODECIDMAP codecidmap_a[] = {
          /* Audio */
         {
+          id: CODEC_ID_MP3,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          decode_parameters: decode_parameters_audio,
+	  short_name: "mp3",
+	  name: "MPEG-1/2 audio layer 1/2/3",
+	  fourccs: {".mp2", ".MP2", "ms\0\x50", "MS\0\x50",".mp3", ".MP3", "ms\0\x55", "MS\0\x55", (char *)0},
+          wav_ids: { 0x50, 0x55, LQT_WAV_ID_NONE },
+        },
+	{
           id: CODEC_ID_MP2,
 	  index: -1,
           encoder: NULL,
@@ -602,20 +678,9 @@ struct CODECIDMAP codecidmap_a[] = {
           decode_parameters: decode_parameters_audio,
 	  short_name: "mp2",
 	  name: "Mpeg Layer 2 Audio",
-	  fourccs: {".mp2", ".MP2", "ms\0\x50", "MS\0\x50", (char *)0},
-          wav_ids: { LQT_WAV_ID_NONE },
-        },
-	{
-          id: CODEC_ID_MP3LAME,
-	  index: -1,
-          encoder: NULL,
-          decoder: NULL,
-          encode_parameters: encode_parameters_audio,
-          decode_parameters: decode_parameters_audio,
-	  short_name: "mp3",
-	  name: "Mpeg Layer 3 Audio",
-	  fourccs: {".mp3", ".MP3", "ms\0\x55", "MS\0\x55", (char *)0},
+	  fourccs: {".mp2", (char *)0 },
           wav_ids: { 0x55, LQT_WAV_ID_NONE },
+          do_encode: 1,
         },
 	{
           id: CODEC_ID_AC3,
@@ -628,8 +693,9 @@ struct CODECIDMAP codecidmap_a[] = {
 	  name: "AC3 Audio",
 	  fourccs: {".ac3", ".AC3", (char *)0},
           wav_ids: { 0x2000, LQT_WAV_ID_NONE },
+          do_encode: 1,
         },
-#if 0 /* Crashes */
+#if 0 /* Sounds ugly */
 	{
           id: CODEC_ID_ADPCM_MS,
 	  index: -1,
@@ -640,6 +706,19 @@ struct CODECIDMAP codecidmap_a[] = {
 	  fourccs: {"ms\0\x02", "MS\0\x02", (char*)0},
           wav_ids: { 0x02, LQT_WAV_ID_NONE },
         },
+#endif
+#if 0 /* Sounds ugly */
+        {
+          id: CODEC_ID_ADPCM_IMA_WAV,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          short_name: "ima adpcm (wav)",
+	  name: "ADPCM ima WAV",
+	  fourccs: {"ms\0\x11", "MS\0\x11", (char*)0},
+          wav_ids: { 0x11, LQT_WAV_ID_NONE },
+          
+        },
 #endif
 	{
           id: CODEC_ID_PCM_ALAW,
@@ -657,12 +736,20 @@ struct CODECIDMAP codecidmap_a[] = {
 #define NUMMAPS_A ((int)(sizeof(codecidmap_a)/sizeof(struct CODECIDMAP)))
 #define NUMMAPS_V ((int)(sizeof(codecidmap_v)/sizeof(struct CODECIDMAP)))
 
-void ffmpeg_map_init(void)
+// static void ffmpeg_map_init(void) __attribute__ ((constructor));
+
+static void ffmpeg_map_init(void)
   {
-  AVCodec *codec;
-  int i, found;
+  int i;
+  //  fprintf(stderr, "Init map...");
   if(ffmpeg_num_video_codecs >= 0)
+    {
+    //    fprintf(stderr, "already initialized\n");
     return;
+    }
+  //  else
+  //    fprintf(stderr, "initializing\n");
+    
   avcodec_register_all();
   avcodec_init();
   ffmpeg_num_video_codecs = 0;
@@ -670,38 +757,45 @@ void ffmpeg_map_init(void)
   
   for(i = 0; i < NUMMAPS_V; i++)
     {
-    codecidmap_v[i].encoder = avcodec_find_encoder(codecidmap_v[i].id);
+    if(codecidmap_v[i].do_encode)
+      codecidmap_v[i].encoder = avcodec_find_encoder(codecidmap_v[i].id);
     codecidmap_v[i].decoder = avcodec_find_decoder(codecidmap_v[i].id);
 
     if(codecidmap_v[i].encoder || codecidmap_v[i].decoder)
-      codecidmap_v[i].index = ffmpeg_num_audio_codecs + ffmpeg_num_video_codecs++;
+      {
+      codecidmap_v[i].index = ffmpeg_num_video_codecs++;
+#if 0
+      fprintf(stderr, "Found video codec %s %p %p %d %s\n",
+              codecidmap_v[i].name, codecidmap_v[i].encoder, codecidmap_v[i].decoder,
+              codecidmap_v[i].index,
+              (codecidmap_v[i].encoder?codecidmap_v[i].encoder->name:codecidmap_v[i].decoder->name));
+#endif
+      }
     }
   for(i = 0; i < NUMMAPS_A; i++)
     {
-    /* UGLY HACK: Disable alaw encoding since it screws up libquicktime */
-    if(codecidmap_a[i].id != CODEC_ID_PCM_ALAW)
+    if(codecidmap_a[i].do_encode)
       codecidmap_a[i].encoder = avcodec_find_encoder(codecidmap_a[i].id);
     codecidmap_a[i].decoder = avcodec_find_decoder(codecidmap_a[i].id);
 
     if(codecidmap_a[i].encoder || codecidmap_a[i].decoder)
       {
       codecidmap_a[i].index = ffmpeg_num_audio_codecs++ + ffmpeg_num_video_codecs;
-      // fprintf(stderr, "Found codec %s %p %p\n", codecidmap_a[i].name, codecidmap_a[i].encoder, codecidmap_a[i].decoder);
+#if 0
+      fprintf(stderr, "Found audio codec %s %p %p %d %s\n",
+              codecidmap_a[i].name, codecidmap_a[i].encoder, codecidmap_a[i].decoder,
+              codecidmap_a[i].index,
+              (codecidmap_a[i].encoder?codecidmap_a[i].encoder->name:codecidmap_a[i].decoder->name));
+#endif
       }
     }
   }
 
-/* Common to all */
-static int encoding_colormodels_ffmpeg[] = {
-	BC_YUV420P, 
-	LQT_COLORMODEL_NONE
-};
-
 /* Template */
 
-static char ffmpeg_name[50];
-static char ffmpeg_long_name[50];
-static char ffmpeg_description[100];
+static char ffmpeg_name[256];
+static char ffmpeg_long_name[256];
+static char ffmpeg_description[256];
 
 static lqt_codec_info_static_t codec_info_ffmpeg = {
 	name:        ffmpeg_name,
@@ -713,7 +807,6 @@ static lqt_codec_info_static_t codec_info_ffmpeg = {
 	direction:   0,
 	encoding_parameters: NULL,
 	decoding_parameters: NULL,
-	encoding_colormodels: encoding_colormodels_ffmpeg,
 	decoding_colormodel:  LQT_COLORMODEL_NONE,
 };
 
@@ -721,6 +814,7 @@ static lqt_codec_info_static_t codec_info_ffmpeg = {
 
 extern int get_num_codecs()
 {
+//        fprintf(stderr, "Get num codecs\n");
 	ffmpeg_map_init();
 	return ffmpeg_num_video_codecs + ffmpeg_num_audio_codecs;
 }
@@ -737,6 +831,7 @@ static void set_codec_info(struct CODECIDMAP * map)
     codec_info_ffmpeg.direction = LQT_DIRECTION_BOTH;
     codec_info_ffmpeg.encoding_parameters = map->encode_parameters;
     codec_info_ffmpeg.decoding_parameters = map->decode_parameters;
+    codec_info_ffmpeg.encoding_colormodels = map->encoding_colormodels;
     //    capabilities = "Codec";
     }
   else if(map->encoder)
@@ -744,6 +839,8 @@ static void set_codec_info(struct CODECIDMAP * map)
     codec_info_ffmpeg.direction = LQT_DIRECTION_ENCODE;
     codec_info_ffmpeg.encoding_parameters = map->encode_parameters;
     codec_info_ffmpeg.decoding_parameters = NULL;
+    codec_info_ffmpeg.encoding_colormodels = map->encoding_colormodels;
+      
     //    capabilities = "Encoder";
     }
   else if(map->decoder)
@@ -754,12 +851,12 @@ static void set_codec_info(struct CODECIDMAP * map)
     //    capabilities = "Decoder";
     }
 
-  snprintf(ffmpeg_name, 50, "ffmpeg_%s", map->short_name);
+  snprintf(ffmpeg_name, 256, "ffmpeg_%s", map->short_name);
   //  snprintf(ffmpeg_long_name, 50, "FFMPEG %s %s", map->name, capabilities);
   //  snprintf(ffmpeg_description, 100, "FFMPEG %s %s", map->name, capabilities);
 
-  snprintf(ffmpeg_long_name, 50, "FFMPEG %s", map->name);
-  snprintf(ffmpeg_description, 100, "FFMPEG %s", map->name);
+  snprintf(ffmpeg_long_name, 256, "FFMPEG %s", map->name);
+  snprintf(ffmpeg_description, 256, "FFMPEG %s", map->name);
 
   
   if((map->encoder && (map->encoder->type == CODEC_TYPE_VIDEO)) ||
@@ -773,6 +870,7 @@ static void set_codec_info(struct CODECIDMAP * map)
 static struct CODECIDMAP * find_codec(int index)
   {
   int i;
+  //  fprintf(stderr, "Find codec\n");
   for(i = 0; i < NUMMAPS_V; i++)
     {
     if(codecidmap_v[i].index == index)
@@ -800,51 +898,6 @@ extern lqt_codec_info_static_t * get_codec_info(int index)
 	return NULL;
 }
 
-/* We obtain the stream colormodel during runtime */
-
-extern int get_stream_colormodel(quicktime_t * file, int track, int codec_index,
-                                 int * exact)
-  {
-  int width, height, depth, ret;
-  quicktime_video_map_t *vtrack;
-  quicktime_trak_t *trak;
-  AVCodecContext *avctx;
-  int dummy;
-  struct CODECIDMAP * map;
-    
-  vtrack = &(file->vtracks[track]);
-  trak = vtrack->track;
-  
-  height = trak->tkhd.track_height;
-  width = trak->tkhd.track_width;
-  depth = quicktime_video_depth(file, track);
-
-  avctx = avcodec_alloc_context();
-
-  avctx->width = width;
-  avctx->height = height;
-  avctx->bits_per_sample = depth;
-  avctx->extradata      = trak->mdia.minf.stbl.stsd.table[0].extradata;
-  avctx->extradata_size = trak->mdia.minf.stbl.stsd.table[0].extradata_size;
-
-  map = find_codec(codec_index);
-  if(!map)
-    {
-    fprintf(stderr, "Found no codec for %d\n", codec_index);
-    return LQT_COLORMODEL_NONE;
-    }
-  if(avcodec_open(avctx, map->decoder) != 0)
-    {
-    fprintf(stderr, "Couldn't open codec No. %d\n", codec_index);
-    return LQT_COLORMODEL_NONE;
-    }
-  ret = lqt_ffmpeg_get_lqt_colormodel(avctx->pix_fmt, &dummy);
-  
-  avcodec_close(avctx);
-  return ret;
-  }
-
-
 /*
  *   Return the actual codec constructor
  */
@@ -860,7 +913,7 @@ void quicktime_init_video_codec_ffmpeg ## x(quicktime_video_map_t *vtrack) \
 	int i; \
 	for(i = 0; i < ffmpeg_num_video_codecs; i++) { \
 		if(codecidmap_v[i].index == x) { \
-			quicktime_init_video_codec_ffmpeg(vtrack, \
+                     quicktime_init_video_codec_ffmpeg(vtrack,       \
 				codecidmap_v[i].encoder, \
 				codecidmap_v[i].decoder); \
 		} \
@@ -904,7 +957,7 @@ void quicktime_init_audio_codec_ffmpeg ## x(quicktime_audio_map_t *atrack) \
 	int i; \
 	for(i = 0; i < ffmpeg_num_audio_codecs; i++) { \
 		if(codecidmap_a[i].index == x) { \
-			quicktime_init_audio_codec_ffmpeg(atrack, \
+                      quicktime_init_audio_codec_ffmpeg(atrack,       \
 				codecidmap_a[i].encoder, \
 				codecidmap_a[i].decoder); \
 		} \
@@ -947,6 +1000,7 @@ IFUNC(29)
 
 extern lqt_init_video_codec_func_t get_video_codec(int index)
 {
+//        fprintf(stderr, "Get video codec %d\n", index);
 	ffmpeg_map_init();
 	if(index > MAX_VIDEO_FUNC) {
 		fprintf(stderr, "lqt_ffmpeg error: Insufficient dummy calls - please report!\n");
@@ -991,6 +1045,7 @@ extern lqt_init_video_codec_func_t get_video_codec(int index)
 
 extern lqt_init_audio_codec_func_t get_audio_codec(int index)
 {
+//        fprintf(stderr, "Get audio codec %d\n", index);
 	ffmpeg_map_init();
 	if(index > MAX_AUDIO_FUNC) {
 		fprintf(stderr, "lqt_ffmpeg error: Insufficient dummy calls - please report!\n");
