@@ -6,7 +6,7 @@
 #include <quicktime/quicktime.h>
 #include <workarounds.h>
 #include "encore50/encore.h"
-#include "decore50/decore.h"
+// #include "decore50/decore.h"
 
 #include <pthread.h>
 #include <stdlib.h>
@@ -18,7 +18,6 @@ typedef struct
 	unsigned char *work_buffer;
 	char *temp_frame;
 	long buffer_size;
-	int decode_initialized;
 	int encode_initialized;
 	int bitrate;
 	long rc_period; // the intended rate control averaging period
@@ -34,20 +33,16 @@ typedef struct
 	long last_frame;  
 	int encode_handle;
 
-        DEC_SET dec_set;
 
-	DEC_PARAM dec_param;
 	ENC_PARAM enc_param;
 
-	int decode_handle;
+//	int decode_handle;
 // Must count pframes in VBR
 	int p_count;
 } quicktime_divx_codec_t;
 
 static pthread_mutex_t encode_mutex;
-static pthread_mutex_t decode_mutex;
 static int mutex_initialized = 0;
-static int decode_handle = 1;
 static int encode_handle = 0;
 
 static int delete_codec(quicktime_video_map_t *vtrack)
@@ -64,45 +59,19 @@ static int delete_codec(quicktime_video_map_t *vtrack)
 			0);
 		pthread_mutex_unlock(&encode_mutex);
 	}
-	if(codec->decode_initialized)
-	{
-		pthread_mutex_lock(&decode_mutex);
-
-
-
-
-
-/*
- * 		decore(codec->handle,
- * 			DEC_OPT_DESTROY,
- * 			0,
- * 			0);
- */
-
-
-
-		decore(codec->decode_handle,
-			DEC_OPT_RELEASE,
-			0,
-			0);
-
-
-
-		pthread_mutex_unlock(&decode_mutex);
-	}
 	if(codec->temp_frame) free(codec->temp_frame);
 	if(codec->work_buffer) free(codec->work_buffer);
 	free(codec);
 	return 0;
 }
-
+#if 0
 static int reads_colormodel(quicktime_t *file, 
 		int colormodel, 
 		int track)
 {
 	return (colormodel == BC_YUV420P);
 }
-
+#endif
 static int writes_colormodel(quicktime_t *file, 
 		int colormodel, 
 		int track)
@@ -134,7 +103,6 @@ static void init_mutex()
 		pthread_mutexattr_t attr;
 		mutex_initialized = 1;
 		pthread_mutexattr_init(&attr);
-		pthread_mutex_init(&decode_mutex, &attr);
 		pthread_mutex_init(&encode_mutex, &attr);
 	}
 }
@@ -241,7 +209,6 @@ int quicktime_divx_write_vol(unsigned char *data_start,
 	unsigned char *data = data_start;
 	int bit_pos;
 	uint64_t bit_store;
-	int i, j;
 
 	bit_store = 0;
 	bit_pos = 0;
@@ -482,278 +449,9 @@ int quicktime_divx_write_vol(unsigned char *data_start,
 }
 
 
-
-
-
-static int decode(quicktime_t *file, unsigned char **row_pointers, int track)
-{
-	int i;
-	int64_t bytes;
-	int result = 0;
-	quicktime_video_map_t *vtrack = &(file->vtracks[track]);
-	quicktime_trak_t *trak = vtrack->track;
-	quicktime_divx_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
-	int width = trak->tkhd.track_width;
-	int height = trak->tkhd.track_height;
-	int width_i = (int)((float)width / 16 + 0.5) * 16;
-	int height_i = (int)((float)height / 16 + 0.5) * 16;
-	DEC_FRAME dec_frame;
-	int use_temp = 0;
-	int input_cmodel;
-	char *bmp_pointers[3];
-	long temp_position;
-
-
-
-
-//printf("decode 1 %d\n", file->color_model);
-
-	init_mutex();
-	pthread_mutex_lock(&decode_mutex);
-
-
-
-
-
-
-
-
-
-
-
-
-	if(!codec->decode_initialized)
-	{
-		DEC_MEM_REQS dec_mem_reqs;
-
-// decore requires handle to be > 1
-		codec->decode_handle = decode_handle++;
-		codec->last_frame = -1;
-		codec->dec_param.x_dim = width_i;
-		codec->dec_param.y_dim = height_i;
-		codec->dec_param.output_format = DEC_420;
-		codec->dec_param.time_incr = 0;
-
-		decore(codec->decode_handle, DEC_OPT_MEMORY_REQS, &codec->dec_param, &dec_mem_reqs);
-		codec->dec_param.buffers.mp4_edged_ref_buffers = calloc(1, dec_mem_reqs.mp4_edged_ref_buffers_size);
-		codec->dec_param.buffers.mp4_edged_for_buffers = calloc(1, dec_mem_reqs.mp4_edged_for_buffers_size);
-		codec->dec_param.buffers.mp4_display_buffers = calloc(1, dec_mem_reqs.mp4_display_buffers_size);
-		codec->dec_param.buffers.mp4_state = calloc(1, dec_mem_reqs.mp4_state_size);
-		codec->dec_param.buffers.mp4_tables = calloc(1, dec_mem_reqs.mp4_tables_size);
-		codec->dec_param.buffers.mp4_stream = calloc(1, dec_mem_reqs.mp4_stream_size);
-                
-                decore(codec->decode_handle, DEC_OPT_INIT, &codec->dec_param, NULL);
-
-                decore(codec->decode_handle, DEC_OPT_SETPP, &codec->dec_set, NULL);
-
-		codec->temp_frame = malloc(width_i * height_i * 3 / 2);
-
-
-/*
- * 		codec->dec_param.width = width_i;
- * 		codec->dec_param.height = height_i;
- * 		decore(0, DEC_OPT_CREATE, &codec->dec_param, NULL);
- */
-
-
-
-
-// Must decode frame with VOL header first but only the first frame in the
-// sequence has a VOL header.
-		temp_position = vtrack->current_position;
-		READ_RAW(0);
-		vtrack->current_position = temp_position;
-		dec_frame.bitstream = codec->work_buffer;
-		dec_frame.length = bytes;
-		dec_frame.stride = width_i;
-		dec_frame.render_flag = 0;
-		dec_frame.bmp[0] = codec->temp_frame;
-		dec_frame.bmp[1] = codec->temp_frame + width_i * height_i;
-		dec_frame.bmp[2] = codec->temp_frame + width_i * height_i * 5 / 4;
-		decore(codec->decode_handle, 0, &dec_frame, NULL);
-
-
-
-		codec->decode_initialized = 1;
-	}
-
-//printf("decode 1\n");
-
-
-
-
-	input_cmodel = BC_YUV420P;
-	if(file->vtracks[track].color_model == input_cmodel &&
-		file->out_w == width_i &&
-		file->out_h == height_i &&
-		file->in_x == 0 &&
-		file->in_y == 0 &&
-		file->in_w == width_i &&
-		file->in_h == height_i)
-	{
-//		dec_frame.dst = row_pointers[0];
-		dec_frame.bmp[0] = row_pointers[0];
-		dec_frame.bmp[1] = row_pointers[1];
-		dec_frame.bmp[2] = row_pointers[2];
-		use_temp = 0;
-	}
-	else
-	{
-//		dec_frame.dst = codec->temp_frame;
-		dec_frame.bmp[0] = codec->temp_frame;
-		dec_frame.bmp[1] = codec->temp_frame + width_i * height_i;
-		dec_frame.bmp[2] = codec->temp_frame + width_i * height_i * 5 / 4;
-		use_temp = 1;
-	}
-
-
-
-
-
-
-
-
-	dec_frame.stride = width_i;
-
-//	dec_frame.render = 1;
-//	dec_frame.colorspace = DEC_420;
-
-
-
-
-
-
-
-
-
-
-
-
-//printf("decode 1 %d %d\n", codec->last_frame, vtrack->current_position);
-
-	if(quicktime_has_keyframes(file, track) && 
-		vtrack->current_position != codec->last_frame + 1)
-	{
-		int frame1, frame2 = vtrack->current_position;
-
-		frame1 = quicktime_get_keyframe_before(file, vtrack->current_position, track);
-
-		if(frame1 < codec->last_frame &&
-			frame2 > codec->last_frame) frame1 = codec->last_frame + 1;
-
-
-		while(frame1 < frame2)
-		{
-			quicktime_set_video_position(file, frame1, track);
-			bytes = quicktime_frame_size(file, frame1, track);
-
-
-			if(!codec->work_buffer || codec->buffer_size < bytes)
-			{
-				if(codec->work_buffer) free(codec->work_buffer);
-				codec->buffer_size = bytes;
-				codec->work_buffer = calloc(1, codec->buffer_size + 100);
-			}
-
-			quicktime_read_data(file, codec->work_buffer, bytes);
-
-//printf("decode 2 %d %d\n", frame1, frame2);
-			dec_frame.bitstream = codec->work_buffer;
-			dec_frame.length = bytes;
-			dec_frame.render_flag = 0;
-			decore(codec->decode_handle, 0, &dec_frame, NULL);
-			frame1++;
-		}
-		
-		
-		vtrack->current_position = frame2;
-	}
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-	codec->last_frame = vtrack->current_position;
-//printf("decode 1\n");
-
-
-	READ_RAW(vtrack->current_position);
-
-
-
-//printf("decode 1\n");
-
-//printf("decode 1\n");
-
-
-	dec_frame.bitstream = codec->work_buffer;
-	dec_frame.length = bytes;
-	dec_frame.render_flag = 1;
-
-//printf("decode 1\n");
-//	decore(codec->dec_param.handle, DEC_OPT_DECODE, &dec_frame, NULL);
-	decore(codec->decode_handle, 0, &dec_frame, NULL);
-//printf("decode 2\n");
-
-
-	pthread_mutex_unlock(&decode_mutex);
-//printf("decode 1 %d %d\n", use_temp, file->color_model);
-
-	if(use_temp)
-	{
-		unsigned char **input_rows = malloc(sizeof(unsigned char*) * height_i);
-		for(i = 0; i < height_i; i++)
-			input_rows[i] = codec->temp_frame + width_i * 3;
-		
-		
-		cmodel_transfer(row_pointers, /* Leave NULL if non existent */
-			input_rows,
-			row_pointers[0], /* Leave NULL if non existent */
-			row_pointers[1],
-			row_pointers[2],
-			codec->temp_frame, /* Leave NULL if non existent */
-			codec->temp_frame + width_i * height_i,
-			codec->temp_frame + width_i * height_i + width_i * height_i / 4,
-			file->in_x,        /* Dimensions to capture from input frame */
-			file->in_y, 
-			file->in_w, 
-			file->in_h,
-			0,       /* Dimensions to project on output frame */
-			0, 
-			file->out_w, 
-			file->out_h,
-			input_cmodel, 
-			file->vtracks[track].color_model,
-			0,         /* When transfering BC_RGBA8888 to non-alpha this is the background color in 0xRRGGBB hex */
-			width_i,       /* For planar use the luma rowspan */
-			width);
-		
-		free(input_rows);
-	}
-
-
-
-
-//printf("decode 2\n");
-
-	return result;
-}
-
-
-
 static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 {
 //printf("encode 1\n");
-	int64_t offset = quicktime_position(file);
 	quicktime_video_map_t *vtrack = &(file->vtracks[track]);
 	quicktime_divx_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
 	quicktime_trak_t *trak = vtrack->track;
@@ -762,7 +460,6 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 	int width_i = (int)((float)width / 16 + 0.5) * 16;
 	int height_i = (int)((float)height / 16 + 0.5) * 16;
 	int result = 0;
-	int i;
         quicktime_atom_t chunk_atom;
 	ENC_FRAME encore_input;
 	ENC_RESULT encore_result;
@@ -924,24 +621,18 @@ static int set_parameter(quicktime_t *file,
 	else
 	if(!strcasecmp(key, "divx_fix_bitrate"))
 		codec->fix_bitrate = *(int*)value;
-	else
-        if(!strcasecmp(key, "divx_postprocess"))
-		codec->dec_set.postproc_level = *(int*)value;
 	return 0;
 }
 
 
 void quicktime_init_codec_divx(quicktime_video_map_t *vtrack)
 {
-	int i;
 	quicktime_divx_codec_t *codec;
 
 /* Init public items */
 	((quicktime_codec_t*)vtrack->codec)->priv = calloc(1, sizeof(quicktime_divx_codec_t));
 	((quicktime_codec_t*)vtrack->codec)->delete_vcodec = delete_codec;
-	((quicktime_codec_t*)vtrack->codec)->decode_video = decode;
 	((quicktime_codec_t*)vtrack->codec)->encode_video = encode;
-	((quicktime_codec_t*)vtrack->codec)->reads_colormodel = reads_colormodel;
 	((quicktime_codec_t*)vtrack->codec)->writes_colormodel = writes_colormodel;
 	((quicktime_codec_t*)vtrack->codec)->set_parameter = set_parameter;
 	
