@@ -92,7 +92,6 @@ int quicktime_atom_read_header(quicktime_t *file, quicktime_atom_t *atom)
 	}
 	else
 	{
-		int64_t size2;
 
 		reset(atom);
 
@@ -207,7 +206,7 @@ void quicktime_atom_write_footer(quicktime_t *file, quicktime_atom_t *atom)
 	quicktime_set_position(file, atom->end);
 }
 
-int quicktime_atom_is(quicktime_atom_t *atom, char *type)
+int quicktime_atom_is(quicktime_atom_t *atom, unsigned char *type)
 {
 	if(atom->type[0] == type[0] &&
 		atom->type[1] == type[1] &&
