@@ -73,15 +73,15 @@ void quicktime_init_hdrl(quicktime_t *file, quicktime_hdrl_t *hdrl)
 	quicktime_write_int32_le(file, 0); /* bitrate in bytes */
     quicktime_write_int32_le(file, 0); /* padding */
     quicktime_write_int32_le(file, 
-		AVI_TRUSTCKTYPE | 
+                             //		AVI_TRUSTCKTYPE | 
 		AVI_HASINDEX | 
-		AVI_MUSTUSEINDEX | 
+                             //		AVI_MUSTUSEINDEX | 
 		AVI_ISINTERLEAVED); /* flags */
 	hdrl->frames_offset = quicktime_position(file);
     quicktime_write_int32_le(file, 0); /* nb frames, filled later */
     quicktime_write_int32_le(file, 0); /* initial frame */
     quicktime_write_int32_le(file, file->moov.total_tracks); /* nb streams */
-    quicktime_write_int32_le(file, 0); /* suggested buffer size */
+    quicktime_write_int32_le(file, 1024 * 1024); /* suggested buffer size */
 
 	if(file->total_vtracks)
     {
@@ -155,12 +155,13 @@ void quicktime_finalize_hdrl(quicktime_t *file, quicktime_hdrl_t *hdrl)
 	for(i = 0; i < file->moov.total_tracks; i++)
 	{
 		quicktime_trak_t *trak = file->moov.trak[i];
-		quicktime_strl_t *strl = hdrl->strl[i];
+		quicktime_strl_t *strl = trak->strl;
 
 		if(trak->mdia.minf.is_video)
 		{
 			int length;
-			quicktime_set_position(file, strl->length_offset);
+                        //                        fprintf(stderr, "Finalize video strl\n");
+			quicktime_set_position(file, strl->dwLengthOffset);
 			total_frames = length = quicktime_track_samples(file, trak);
 			quicktime_write_int32_le(file, length);
 			frame_rate = (double)trak->mdia.mdhd.time_scale /
@@ -169,22 +170,32 @@ void quicktime_finalize_hdrl(quicktime_t *file, quicktime_hdrl_t *hdrl)
 		else
 		if(trak->mdia.minf.is_audio)
 		{
-			int length, samples_per_chunk;
-			quicktime_set_position(file, strl->length_offset);
+			int length;
+#if 0
+                        fprintf(stderr, "Finalize audio strl %lld %lld %lld %lld\n",
+                                strl->dwLengthOffset,
+                                strl->dwScaleOffset,
+                                strl->dwSampleSizeOffset,
+                                strl->nAvgBytesPerSecOffset
+                                );
+#endif
+			quicktime_set_position(file, strl->dwLengthOffset);
 			length = quicktime_track_samples(file, trak);
 			quicktime_write_int32_le(file, length);
-			quicktime_set_position(file, strl->samples_per_chunk_offset);
+			quicktime_set_position(file, strl->dwScaleOffset);
 
-			samples_per_chunk = quicktime_avg_chunk_samples(file, trak);
-			quicktime_write_int32_le(file, 
-				samples_per_chunk);
-			quicktime_write_int32_le(file, 
-				samples_per_chunk *
-				trak->mdia.minf.stbl.stsd.table[0].sample_rate);
+			quicktime_write_int32_le(file, strl->dwScale);
+			quicktime_write_int32_le(file, strl->dwRate);
 
-			quicktime_set_position(file, strl->sample_size_offset);
-			quicktime_write_int32_le(file, 
-				trak->mdia.minf.stbl.stsd.table[0].sample_size);
+			quicktime_set_position(file, strl->dwSampleSizeOffset);
+			quicktime_write_int32_le(file, strl->dwSampleSize);
+
+                        quicktime_set_position(file, strl->nAvgBytesPerSecOffset);
+                        quicktime_write_int32_le(file, strl->nAvgBytesPerSec);
+			quicktime_write_int16_le(file, strl->nBlockAlign);
+                        quicktime_write_int16_le(file, strl->wBitsPerSample);
+                        
+                        
 		}
 	}
 
