@@ -12,7 +12,7 @@ typedef struct
 } avi_tag_t;
 
 
-
+#if 0
 static int is_keyframe(quicktime_trak_t *trak, int frame)
 {
 	int i;
@@ -24,7 +24,7 @@ static int is_keyframe(quicktime_trak_t *trak, int frame)
 	}
 	return 0;
 }
-
+#endif
 
 void quicktime_delete_idx1(quicktime_idx1_t *idx1)
 {
@@ -36,8 +36,6 @@ void quicktime_read_idx1(quicktime_t *file,
 	quicktime_atom_t *parent_atom)
 {
 	int i;
-	quicktime_riff_t *first_riff = file->riff[0];
-	quicktime_hdrl_t *hdrl = &first_riff->hdrl;
 	quicktime_idx1_t *idx1 = &riff->idx1;
 
 //printf("quicktime_read_idx1 1 %llx\n", quicktime_position(file));
@@ -63,81 +61,6 @@ void quicktime_read_idx1(quicktime_t *file,
 //printf("quicktime_read_idx1 100\n");
 
 
-
-
-#if 0
-	while(quicktime_position(file) <= parent_atom->end - 0x10)
-	{
-		int current_vtrack = 0;
-		int current_atrack = 0;
-		quicktime_read_data(file, data, 4);
-
-		for(i = 0; i < file->moov.total_tracks; i++)
-		{
-			if(quicktime_match_24(data, tags[i]))
-			{
-// Flags
-				int flags = quicktime_read_int32_le(file);
-				int64_t offset = quicktime_read_int32_le(file);
-
-// Offset from start of movi
-				quicktime_update_stco(&file->moov.trak[i]->mdia.minf.stbl.stco, 
-					file->moov.trak[i]->mdia.minf.stbl.stco.total_entries + 1, 
-					offset);
-
-				if(file->moov.trak[i]->mdia.minf.is_audio)
-				{
-					long chunk_size;
-
-// Bytes in chunk
-					chunk_size = quicktime_read_int32_le(file);
-
-
-// Update samples per chunk table if PCM
-					if(file->moov.trak[i]->mdia.minf.stbl.stsd.table[0].packet_size > 0)
-					{
-						quicktime_update_stsc(&file->moov.trak[i]->mdia.minf.stbl.stsc, 
-							file->moov.trak[i]->mdia.minf.stbl.stsc.total_entries + 1, 
-							chunk_size / 
-								file->moov.trak[i]->mdia.minf.stbl.stsd.table[0].packet_size);
-					}
-				}
-				else
-				{
-					long sample_size;
-
-// Bytes in chunk
-					sample_size = quicktime_read_int32_le(file);
-
-//printf("quicktime_read_idx1 5 %d %d\n", file->moov.trak[i]->mdia.minf.stbl.stsz.total_entries, current_vtrack);
-
-// Update keyframe table
-					if((flags & AVI_KEYFRAME) == AVI_KEYFRAME)
-						quicktime_insert_keyframe(file, 
-							file->moov.trak[i]->mdia.minf.stbl.stsz.total_entries, 
-							current_vtrack);
-//printf("quicktime_read_idx1 6\n");
-
-// Update samplesize table
-					quicktime_update_stsz(&file->moov.trak[i]->mdia.minf.stbl.stsz, 
-						file->moov.trak[i]->mdia.minf.stbl.stsz.total_entries, 
-						sample_size);
-//printf("quicktime_read_idx1 7\n");
-				}
-			}
-
-			if(file->moov.trak[i]->mdia.minf.is_audio)
-				current_atrack++;
-			else
-				current_vtrack++;
-		}
-	}
-
-	quicktime_atom_skip(file, parent_atom);
-#endif
-
-
-
 }
 
 void quicktime_write_idx1(quicktime_t *file, 
@@ -176,7 +99,6 @@ void quicktime_set_idx1_keyframe(quicktime_t *file,
 	quicktime_idx1_t *idx1 = &riff->idx1;
 	int i;
 	int counter = -1;
-
 // Search through entire index for right numbered tag.
 // Since all the tracks are combined in the same index, this is unavoidable.
 	for(i = 0; i < idx1->table_size; i++)
@@ -206,9 +128,7 @@ void quicktime_update_idx1table(quicktime_t *file,
 	quicktime_idx1_t *idx1 = &riff->idx1;
 	quicktime_movi_t *movi = &riff->movi;
 	quicktime_idx1table_t *idx1_table;
-	quicktime_stss_t *stss = &trak->mdia.minf.stbl.stss;
-	uint32_t flags = 0;
-	int i;
+#if 0
 	int keyframe_frame = idx1->table_size + 1;
 
 // Set flag for keyframe
@@ -225,7 +145,7 @@ void quicktime_update_idx1table(quicktime_t *file,
 			break;
 		}
 	}
-
+#endif
 
 // Allocation
 	if(idx1->table_size >= idx1->table_allocation)
@@ -246,7 +166,7 @@ void quicktime_update_idx1table(quicktime_t *file,
 // Appendage
 	idx1_table = &idx1->table[idx1->table_size];
 	memcpy(idx1_table->tag, tag, 4);
-	idx1_table->flags = flags;
+	idx1_table->flags = (trak->mdia.minf.is_audio) ? AVI_KEYFRAME : 0;
 	idx1_table->offset = offset - 8 - movi->atom.start;
 	idx1_table->size = size;
 	idx1->table_size++;
