@@ -815,7 +815,7 @@
 			switch(out_colormodel) \
 			{ \
 				case BC_TRANSPARENCY: \
-					TRANSFER_FRAME_HEAD \
+					TRANSFER_FRAME_HEAD_BITCOUNTER \
 					transfer_RGBA8888_to_TRANSPARENCY((output), (input), &bit_counter); \
 					TRANSFER_FRAME_TAIL \
 					break; \
@@ -1133,6 +1133,15 @@
 					transfer_BGR8888_to_BGR8888((output), (input));    \
 					TRANSFER_FRAME_TAIL \
 					break; \
+      				case BC_YUV420P: \
+					TRANSFER_YUV420P_OUT_HEAD \
+					transfer_BGR888_to_YUV420P_YUV422P(output_y, \
+						output_u, \
+						output_v, \
+						(input), \
+						j); \
+					TRANSFER_FRAME_TAIL \
+					break; \
 			} \
 			break; \
  \
@@ -1144,7 +1153,124 @@
 					transfer_BGR888_to_RGB888((output), (input));    \
 					TRANSFER_FRAME_TAIL \
 					break; \
-			} \
+				case BC_RGB8: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_RGB8((output), (input));      \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_BGR565: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_BGR565((output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_RGB565: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_RGB565((output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_BGR888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_BGR888((output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_RGBA8888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_RGBA8888((output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_RGB161616: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_RGB161616((uint16_t**)(output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_RGBA16161616: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_RGBA16161616((uint16_t**)(output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_ARGB8888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_ARGB8888((output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_ABGR8888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_ABGR8888((output), (input));    \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_BGR8888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_BGR8888((output), (input));   \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUV888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_YUV888((output), (input));   \
+					TRANSFER_FRAME_TAIL \
+					break; \
+                                case BC_VYU888: \
+                                        TRANSFER_FRAME_HEAD \
+                                        transfer_BGR888_to_VYU888((output), (input));   \
+                                        TRANSFER_FRAME_TAIL \
+                                        break; \
+				case BC_YUVA8888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_YUVA8888((output), (input));   \
+                                        TRANSFER_FRAME_TAIL \
+                                        break; \
+				case BC_YUV161616: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_YUV161616((uint16_t**)(output), (input));   \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUVA16161616: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_YUVA16161616((uint16_t**)(output), (input));   \
+					TRANSFER_FRAME_TAIL \
+					break; \
+                                case BC_UYVA8888: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_UYVA8888((output), (input));   \
+                                        TRANSFER_FRAME_TAIL \
+					break; \
+                                case BC_YUV101010: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_YUV101010((output), (input));   \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUV420P: \
+					TRANSFER_YUV420P_OUT_HEAD \
+					transfer_BGR888_to_YUV420P_YUV422P(output_y, \
+						output_u, \
+						output_v, \
+						(input), \
+						j); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUV422: \
+					TRANSFER_FRAME_HEAD \
+					transfer_BGR888_to_YUV422((output), (input), j); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUV422P: \
+					TRANSFER_YUV422P_OUT_HEAD \
+					transfer_BGR888_to_YUV420P_YUV422P(output_y, \
+						output_u, \
+						output_v, \
+						(input), \
+						j); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+				case BC_YUV444P: \
+					TRANSFER_YUV444P_OUT_HEAD \
+					transfer_BGR888_to_YUV444P(output_y, \
+						output_u, \
+						output_v, \
+						(input), \
+						j); \
+					TRANSFER_FRAME_TAIL \
+					break; \
+			}      \
 			break; \
 	} \
 }
