@@ -40,14 +40,13 @@ static int quicktime_encode_video_stub(quicktime_t *file,
 }
 
 static int quicktime_decode_audio_stub(quicktime_t *file, 
-					int16_t *output_i, 
-					float *output_f, 
-					long samples, 
-					int track, 
-					int channel)
+                                       int16_t ** output_i, 
+                                       float ** output_f, 
+                                       long samples,
+                                       int track)
 {
 	printf("quicktime_decode_audio_stub called\n");
-	return 1;
+	return 0;
 }
 
 static int quicktime_encode_audio_stub(quicktime_t *file, 
@@ -463,7 +462,7 @@ int quicktime_supported_audio(quicktime_t *file, int track)
         return 1;
 }
 
-static void update_frame_position(quicktime_video_map_t * track)
+void lqt_update_frame_position(quicktime_video_map_t * track)
   {
   track->timestamp +=
     track->track->mdia.minf.stbl.stts.table[track->stts_index].sample_duration;
@@ -483,9 +482,12 @@ int quicktime_decode_video(quicktime_t *file,
                            unsigned char **row_pointers, int track)
 {
 	int result;
-	quicktime_trak_t *trak = file->vtracks[track].track;
-	int track_height = trak->tkhd.track_height;
-	int track_width = trak->tkhd.track_width;
+        int track_height;
+	int track_width;
+
+        
+	track_height = quicktime_video_height(file, track);
+	track_width =  quicktime_video_width(file, track);
 
 //printf("quicktime_decode_video 1\n");
 // Fake scaling parameters
@@ -501,7 +503,7 @@ int quicktime_decode_video(quicktime_t *file,
 
         //printf("quicktime_decode_video 1\n");
 	result = ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
-        update_frame_position(&(file->vtracks[track]));
+        lqt_update_frame_position(&(file->vtracks[track]));
 //printf("quicktime_decode_video 2\n");
 	return result;
 }
@@ -513,10 +515,13 @@ int quicktime_decode_video(quicktime_t *file,
 int lqt_decode_video(quicktime_t *file,
                      unsigned char **row_pointers, int track)
 {
-	int result;
-	quicktime_trak_t *trak = file->vtracks[track].track;
-	int track_height = trak->tkhd.track_height;
-	int track_width = trak->tkhd.track_width;
+        int result;
+        int track_height;
+	int track_width;
+
+        
+	track_height = quicktime_video_height(file, track);
+	track_width =  quicktime_video_width(file, track);
 
 //printf("quicktime_decode_video 1\n");
 // Fake scaling parameters
@@ -531,7 +536,7 @@ int lqt_decode_video(quicktime_t *file,
 //printf("quicktime_decode_video 1\n");
 	result =
           ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
-        update_frame_position(&(file->vtracks[track]));
+        lqt_update_frame_position(&(file->vtracks[track]));
 //printf("quicktime_decode_video 2\n");
 	return result;
 }
@@ -559,7 +564,7 @@ long quicktime_decode_scaled(quicktime_t *file,
 	file->out_h = out_h;
 
 	result = ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
-        update_frame_position(&(file->vtracks[track]));
+        lqt_update_frame_position(&(file->vtracks[track]));
 	return result;
 }
 
@@ -572,30 +577,79 @@ int quicktime_encode_video(quicktime_t *file,
 //printf("quicktime_encode_video 1 %p\n", ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video);
 	result = ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video(file, row_pointers, track);
 //printf("quicktime_encode_video 2\n");
-	file->vtracks[track].current_position++;
+        quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
+                              file->vtracks[track].current_position, 0);
+        file->vtracks[track].current_position++;
+	return result;
+}
+
+int lqt_encode_video(quicktime_t *file, 
+                     unsigned char **row_pointers, 
+                     int track, int64_t time)
+{
+	int result;
+//printf("quicktime_encode_video 1 %p\n", ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video);
+	result = ((quicktime_codec_t*)file->vtracks[track].codec)->encode_video(file, row_pointers, track);
+//printf("quicktime_encode_video 2\n");
+
+        if(file->vtracks[track].current_position)
+          quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
+                                file->vtracks[track].current_position - 1,
+                                time - file->vtracks[track].timestamp);
+        else
+          quicktime_update_stts(&file->vtracks[track].track->mdia.minf.stbl.stts,
+                                file->vtracks[track].current_position, 0);
+
+        file->vtracks[track].timestamp = time;
+        file->vtracks[track].current_position++;
 	return result;
 }
 
 
+
 int quicktime_decode_audio(quicktime_t *file, 
-				int16_t *output_i, 
-				float *output_f, 
-				long samples, 
-				int channel)
+                           int16_t *output_i, 
+                           float *output_f, 
+                           long samples, 
+                           int channel)
 {
-	int quicktime_track, quicktime_channel;
+        float   ** channels_f;
+        int16_t ** channels_i;
+
+        int quicktime_track, quicktime_channel;
 	int result = 1;
 
-	quicktime_channel_location(file, &quicktime_track, &quicktime_channel, channel);
-	result = ((quicktime_codec_t*)file->atracks[quicktime_track].codec)->decode_audio(file, 
-				output_i, 
-				output_f, 
-				samples, 
-				quicktime_track, 
-				quicktime_channel);
-	file->atracks[quicktime_track].current_position += samples;
+        quicktime_channel_location(file, &quicktime_track,
+                                   &quicktime_channel, channel);
 
-	return result;
+        if(output_i)
+          {
+          channels_i = calloc(quicktime_track_channels(file, quicktime_track), sizeof(*channels_i));
+          channels_i[quicktime_channel] = output_i;
+          }
+        else
+          channels_i = (int16_t**)0;
+        
+        if(output_f)
+          {
+          channels_f = calloc(quicktime_track_channels(file, quicktime_track), sizeof(*channels_f));
+          channels_f[quicktime_channel] = output_f;
+          }
+        else
+          channels_f = (float**)0;
+        
+	result = ((quicktime_codec_t*)file->atracks[quicktime_track].codec)->decode_audio(file, 
+                                                                                          channels_i, 
+                                                                                          channels_f, 
+                                                                                          samples,
+                                                                                          quicktime_track);
+	file->atracks[quicktime_track].current_position += result;
+
+        if(channels_i)
+          free(channels_i);
+        else if(channels_f)
+          free(channels_f);
+	return ((result < 0) ? 1 : 0);
 }
 
 int lqt_total_channels(quicktime_t *file)
@@ -621,45 +675,42 @@ int lqt_decode_audio(quicktime_t *file,
 				float **poutput_f, 
 				long samples)
 {
-	int quicktime_track, quicktime_channel;
 	int result = 1;
-	int total_channels = lqt_total_channels(file);
 	int i = 0;
-	int16_t *output_i;
-	float *output_f;
+	int16_t **output_i;
+	float   **output_f;
 
-	for( i=0; i < total_channels; i++ )
-	{
-		quicktime_channel_location( file, &quicktime_track, &quicktime_channel,
-									i );
+	int total_tracks = quicktime_audio_tracks(file);
+        int track_channels;
 
-		if( poutput_i != NULL )
-		{
-			output_i = poutput_i[i];
-		}
-		else
-			output_i = NULL;
-		
-		if( poutput_f != NULL )
-		{
-			output_f = poutput_f[i];
-		}
-		else
-			output_f = NULL;
+        if(poutput_i)
+          output_i = poutput_i;
+        else
+          output_i = (int16_t**)0;
 
-		if( output_i != NULL || output_f != NULL )
-		{	
-			result = ((quicktime_codec_t*)file->atracks[quicktime_track].codec)->decode_audio(
-				file, 
-				output_i, 
-				output_f, 
-				samples, 
-				quicktime_track, 
-				quicktime_channel );
-		}
-	}
+        if(poutput_f)
+          output_f = poutput_f;
+        else
+          output_f = (float**)0;
+        
+	for( i=0; i < total_tracks; i++ )
+          {
+          track_channels = quicktime_track_channels(file, i);
+
+          result = ((quicktime_codec_t*)file->atracks[i].codec)->decode_audio(
+                                                                              file, 
+                                                                              output_i, 
+                                                                              output_f, 
+                                                                              samples, 
+                                                                              i);
+          if(output_f)
+            output_f += track_channels;
+          if(output_i)
+            output_i += track_channels;
+
+          file->atracks[i].current_position += samples;
+          }
 
-	file->atracks[quicktime_track].current_position += samples;
 
 	return result;
 }
@@ -671,65 +722,40 @@ int lqt_decode_audio_track(quicktime_t *file,
                            int track)
   {
   int result = 1;
-  int16_t *output_i;
-  float *output_f;
-  int i;
-  int total_channels = quicktime_track_channels(file, track);
+  //  fprintf(stderr, "lqt_decode_audio_track\n");
   
-  for( i=0; i < total_channels; i++ )
-    {
-    
-    if( poutput_i != NULL )
-      {
-      output_i = poutput_i[i];
-      }
-    else
-      output_i = NULL;
-    
-    if( poutput_f != NULL )
-      {
-      output_f = poutput_f[i];
-      }
-    else
-      output_f = NULL;
-    
-    if( output_i != NULL || output_f != NULL )
-      {	
-      result = ((quicktime_codec_t*)file->atracks[track].codec)->decode_audio(
-                                                                              file, 
-                                                                              output_i, 
-                                                                              output_f, 
-                                                                              samples, 
-                                                                              track, 
-                                                                              i );
-      }
-    }
+  result = !(((quicktime_codec_t*)file->atracks[track].codec)->decode_audio(
+                                                                           file, 
+                                                                           poutput_i, 
+                                                                           poutput_f, 
+                                                                           samples, 
+                                                                           track));
   
   file->atracks[track].current_position += samples;
   
   return result;
   }
 
-
-
-/* Since all channels are written at the same time: */
-/* Encode using the compressor for the first audio track. */
-/* Which means all the audio channels must be on the same track. */
-
-int quicktime_encode_audio(quicktime_t *file, int16_t **input_i, float **input_f, long samples)
-{
+int lqt_encode_audio_track(quicktime_t *file, 
+                           int16_t **input_i, 
+                           float **input_f, 
+                           long samples,
+                           int track)
+  {
 	int result = 1;
-	char *compressor = quicktime_audio_compressor(file, 0);
 
-	result = ((quicktime_codec_t*)file->atracks[0].codec)->encode_audio(file, 
-		input_i, 
-		input_f,
-		0, 
-		samples);
-	file->atracks[0].current_position += samples;
+	result = ((quicktime_codec_t*)file->atracks[track].codec)->encode_audio(file, input_i, 
+                                                                            input_f, track, samples);
+	file->atracks[track].current_position += samples;
 
 	return result;
-}
+  
+  }
+
+int quicktime_encode_audio(quicktime_t *file, int16_t **input_i, float **input_f, long samples)
+  {
+  return lqt_encode_audio_track(file, input_i, input_f, samples, 0);
+  }
 
 int quicktime_reads_cmodel(quicktime_t *file, 
 	int colormodel, 
@@ -761,6 +787,7 @@ void quicktime_flush_vcodec(quicktime_t *file, int track)
 	((quicktime_codec_t*)file->vtracks[track].codec)->flush(file, track);
 }
 
+#if 0
 int64_t quicktime_samples_to_bytes(quicktime_trak_t *track, long samples)
 {
 	char *compressor = track->mdia.minf.stbl.stsd.table[0].format;
@@ -775,7 +802,7 @@ int64_t quicktime_samples_to_bytes(quicktime_trak_t *track, long samples)
 /* Default use the sample size specification for TWOS and RAW */
 	return samples * channels * track->mdia.minf.stbl.stsd.table[0].sample_size / 8;
 }
-
+#endif
 int quicktime_codecs_flush(quicktime_t *file)
 {
 	int result = 0;
@@ -799,3 +826,63 @@ int quicktime_codecs_flush(quicktime_t *file)
 	}
 	return result;
 }
+
+/* Copy audio */
+
+int lqt_copy_audio(int16_t ** dst_i, float ** dst_f,
+                   int16_t ** src_i, float ** src_f,
+                   int dst_pos, int src_pos,
+                   int dst_size, int src_size, int num_channels)
+  {
+  
+  int i, j, i_tmp;
+  int samples_to_copy;
+  samples_to_copy = src_size < dst_size ? src_size : dst_size;
+
+  //  fprintf(stderr, "lqt copy audio %d, src_pos: %d, dst_pos: %d, src_size: %d, dst_size: %d, samples_to_copy: %d\n",
+  //          num_channels, src_pos, dst_pos, src_size, dst_size, samples_to_copy);
+  
+  if(src_i)
+    {
+    for(i = 0; i < num_channels; i++)
+      {
+      if(dst_i && dst_i[i]) /* int -> int */
+        {
+        memcpy(dst_i[i] + dst_pos, src_i[i] + src_pos, samples_to_copy * sizeof(int16_t));
+        }
+      if(dst_f && dst_f[i]) /* int -> float */
+        {
+        for(j = 0; j < samples_to_copy; j++)
+          {
+          dst_f[i][dst_pos + j] = (float)src_i[i][src_pos + j] / 32767.0;
+          }
+        }
+      }
+    }
+  else if(src_f)
+    {
+    for(i = 0; i < num_channels; i++)
+      {
+      if(dst_i && dst_i[i]) /* float -> int */
+        {
+        for(j = 0; j < samples_to_copy; j++)
+          {
+          i_tmp = (int)(src_f[i][src_pos + j] * 32767.0);
+          
+          if(i_tmp > 32767)
+            i_tmp = 32767;
+
+          if(i_tmp < -32768)
+            i_tmp = -32768;
+          
+          dst_i[i][dst_pos + j] = i_tmp;
+          }
+        }
+      if(dst_f && dst_f[i]) /* float -> float */
+        {
+        memcpy(dst_f[i] + dst_pos, src_f[i] + src_pos, samples_to_copy * sizeof(float));
+        }
+      }
+    }
+  return samples_to_copy;
+  }
