@@ -1,22 +1,27 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <quicktime/colormodels.h>
 #include <quicktime/quicktime.h>
 #include <quicktime/lqt.h>
+#include <lqt_codecinfo_private.h>
+
 #include <lqt_private.h>
 #include <funcprotos.h>
 #include <lqt_fseek.h>
 #include <sys/stat.h>
 #include <string.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 static int64_t get_file_length(quicktime_t *file)
 {
-	struct stat status;
-	if(fstat(fileno(file->stream), &status))
-		perror("get_file_length fstat:");
-	return status.st_size;
+        int64_t current_pos, total_bytes;
+        current_pos = ftello(file->stream);
+        fseeko(file->stream, 0, SEEK_END);
+        total_bytes = ftello(file->stream);
+        fseeko(file->stream, current_pos, SEEK_CUR);
+        return total_bytes;
 }
 
 int quicktime_make_streamable(char *in_path, char *out_path)
@@ -27,6 +32,8 @@ int quicktime_make_streamable(char *in_path, char *out_path)
 	quicktime_atom_t leaf_atom;
 	int64_t moov_length = 0;
 	
+	memset(&new_file,0,sizeof(quicktime_t));
+	
 	quicktime_init(&file);
 
 /* find the moov atom in the old file */
@@ -64,8 +71,8 @@ int quicktime_make_streamable(char *in_path, char *out_path)
 
 			atoms++;
 		}
-	}while(!result && quicktime_position(&file) < file.total_length);
-
+	}while(!result && quicktime_position(&file) <
+               file.total_length);
 	fclose(file.stream);
 
 	if(!moov_exists)
@@ -97,9 +104,11 @@ int quicktime_make_streamable(char *in_path, char *out_path)
 				return 1;
 			}
 
-			quicktime_shift_offsets(&(old_file->moov), moov_length);
+			quicktime_shift_offsets(&(old_file->moov), moov_length+8);
 
 /* open the output file */
+			
+			
 			if(!(new_file.stream = fopen(out_path, "wb")))
 			{
 				perror("quicktime_make_streamable");
@@ -110,7 +119,18 @@ int quicktime_make_streamable(char *in_path, char *out_path)
 /* set up some flags */
 				new_file.wr = 1;
 				new_file.rd = 0;
+				new_file.cpus = 1;
+		      	new_file.presave_buffer = calloc(1, QUICKTIME_PRESAVE);
+
 				quicktime_write_moov(&new_file, &(old_file->moov));
+				
+				quicktime_set_position(&new_file, moov_length);
+
+        	    quicktime_atom_write_header64(&new_file, 
+                                         
+&(new_file.mdat.atom), 
+                                          "mdat");
+				
 				quicktime_set_position(old_file, mdat_start);
 
 				if(!(buffer = calloc(1, buf_size)))
@@ -120,19 +140,40 @@ int quicktime_make_streamable(char *in_path, char *out_path)
 				}
 				else
 				{
-					while(quicktime_position(old_file) < mdat_start + mdat_size && !result)
+					while(quicktime_position(old_file) < mdat_start +
+mdat_size && !result)
 					{
-						if(quicktime_position(old_file) + buf_size > mdat_start + mdat_size)
-							buf_size = mdat_start + mdat_size - quicktime_position(old_file);
+						if(quicktime_position(old_file) + buf_size >
+mdat_start + mdat_size)
+							buf_size = mdat_start + mdat_size -
+quicktime_position(old_file);
 
-						if(!quicktime_read_data(old_file, buffer, buf_size)) result = 1;
+						if(!quicktime_read_data(old_file, buffer, buf_size))
+result = 1;
 						if(!result)
 						{
-							if(!quicktime_write_data(&new_file, buffer, buf_size)) result = 1;
+							if(!quicktime_write_data(&new_file, buffer,
+buf_size)) result = 1;
+ 
 						}
 					}
 					free(buffer);
 				}
+
+				quicktime_atom_write_footer(&new_file, 
+                                         
+&(new_file.mdat.atom));
+			
+				
+       			if(new_file.presave_size)
+        		{
+        			quicktime_fseek(&new_file,
+new_file.presave_position - new_file.presave_size);
+                	fwrite(new_file.presave_buffer, 1,
+new_file.presave_size, new_file.stream);
+                	new_file.presave_size = 0;
+        		}
+				free(new_file.presave_buffer);
 				fclose(new_file.stream);
 			}
 			quicktime_close(old_file);
@@ -147,22 +188,37 @@ int quicktime_make_streamable(char *in_path, char *out_path)
 	return 0;
 }
 
+
+
+
+void lqt_set_audio_parameter(quicktime_t *file,int stream, char *key,void *value)
+  {
+  quicktime_codec_t *codec = (quicktime_codec_t*)file->atracks[stream].codec;
+  if(codec->set_parameter) codec->set_parameter(file, stream, key, value);
+  }
+
+void lqt_set_video_parameter(quicktime_t *file,int stream, char *key,void *value)
+  {
+  quicktime_codec_t *codec = (quicktime_codec_t*)file->vtracks[stream].codec;
+  if(codec->set_parameter) codec->set_parameter(file, stream, key, value);
+  }
+
 void quicktime_set_parameter(quicktime_t *file, char *key, void *value)
 {
 	int i;
 	for(i = 0; i < file->total_vtracks; i++)
 	{
-		quicktime_codec_t *codec = (quicktime_codec_t*)file->vtracks[i].codec;
-		if(codec->set_parameter) codec->set_parameter(file, i, key, value);
+        lqt_set_video_parameter(file,i, key,value);
 	}
 
 	for(i = 0; i < file->total_atracks; i++)
 	{
-		quicktime_codec_t *codec = (quicktime_codec_t*)file->atracks[i].codec;
-		if(codec->set_parameter) codec->set_parameter(file, i, key, value);
+        lqt_set_audio_parameter(file,i, key,value);
 	}
 }
 
+
+
 void quicktime_set_jpeg(quicktime_t *file, int quality, int use_float)
 {
 	quicktime_set_parameter( file, "jpeg_quality", &quality );
@@ -217,6 +273,67 @@ char* quicktime_get_info(quicktime_t *file)
 	return file->moov.udta.info;
 }
 
+/* Extended metadata support */
+
+void lqt_set_album(quicktime_t *file, char *string)
+{
+	quicktime_set_udta_string(&(file->moov.udta.album), &(file->moov.udta.album_len), string);
+}
+
+void lqt_set_artist(quicktime_t *file, char *string)
+{
+	quicktime_set_udta_string(&(file->moov.udta.artist), &(file->moov.udta.artist_len), string);
+}
+
+void lqt_set_genre(quicktime_t *file, char *string)
+{
+	quicktime_set_udta_string(&(file->moov.udta.genre), &(file->moov.udta.genre_len), string);
+}
+
+void lqt_set_track(quicktime_t *file, char *string)
+{
+	quicktime_set_udta_string(&(file->moov.udta.track), &(file->moov.udta.track_len), string);
+}
+
+void lqt_set_comment(quicktime_t *file, char *string)
+{
+	quicktime_set_udta_string(&(file->moov.udta.comment), &(file->moov.udta.comment_len), string);
+}
+
+void lqt_set_author(quicktime_t *file, char *string)
+{
+	quicktime_set_udta_string(&(file->moov.udta.author), &(file->moov.udta.author_len), string);
+}
+
+char * lqt_get_album(quicktime_t * file)
+{
+        return file->moov.udta.album;
+}
+
+char * lqt_get_artist(quicktime_t * file)
+{
+        return file->moov.udta.artist;
+}
+
+char * lqt_get_genre(quicktime_t * file)
+{
+        return file->moov.udta.genre;
+}
+
+char * lqt_get_track(quicktime_t * file)
+{
+        return file->moov.udta.track;
+}
+
+char * lqt_get_comment(quicktime_t *file)
+{
+        return file->moov.udta.comment;
+}
+
+char * lqt_get_author(quicktime_t *file)
+{
+        return file->moov.udta.author;
+}
 
 int quicktime_video_tracks(quicktime_t *file)
 {
@@ -241,70 +358,73 @@ int quicktime_audio_tracks(quicktime_t *file)
 	return result;
 }
 
+int lqt_add_audio_track(quicktime_t *file,
+                        int channels, long sample_rate, int bits, lqt_codec_info_t * codec_info)
+  {
+  quicktime_trak_t *trak;
+  char * compressor = codec_info->fourccs[0];
+
+  /* Fake the bits parameter for some formats. */
+  if(quicktime_match_32(compressor, QUICKTIME_ULAW) ||
+     quicktime_match_32(compressor, QUICKTIME_IMA4))
+    bits = 16;
+  else if(quicktime_match_32(compressor, QUICKTIME_RAW))
+    bits = 8;
+
+  file->atracks = realloc(file->atracks, (file->total_atracks+1)*sizeof(quicktime_audio_map_t));
+  memset(&(file->atracks[file->total_atracks]), 0, sizeof(quicktime_audio_map_t));
+
+  trak = quicktime_add_track(file);
+  quicktime_trak_init_audio(file, trak, channels,
+                            sample_rate, bits, compressor);
+  
+  quicktime_init_audio_map(&(file->atracks[0]), trak, file->wr,
+                           codec_info);
+  file->atracks[file->total_atracks].track = trak;
+  file->atracks[file->total_atracks].channels = channels;
+  file->atracks[file->total_atracks].current_position = 0;
+  file->atracks[file->total_atracks].current_chunk = 1;
+  lqt_set_default_audio_parameters(file, file->total_atracks);
+  file->total_atracks++;
+  return 0;
+  }
+
+int lqt_set_audio(quicktime_t *file, int channels,
+                  long sample_rate,  int bits,
+                  lqt_codec_info_t * codec_info)
+  {
+  lqt_add_audio_track(file, channels, sample_rate, bits, codec_info);
+  return 0;
+  }
+
 int quicktime_set_audio(quicktime_t *file, 
 						int channels,
 						long sample_rate,
 						int bits,
 						char *compressor)
 {
-	quicktime_trak_t *trak;
-
-/* allocate an arbitrary number of tracks */
-	if(channels)
-	{
-/* Fake the bits parameter for some formats. */
-		if(quicktime_match_32(compressor, QUICKTIME_ULAW) ||
-			quicktime_match_32(compressor, QUICKTIME_IMA4)) bits = 16;
-
-		file->atracks = (quicktime_audio_map_t*)calloc(1, sizeof(quicktime_audio_map_t));
-		trak = quicktime_add_track(file);
-		quicktime_trak_init_audio(file, trak, channels, sample_rate, bits, compressor);
-		quicktime_init_audio_map(&(file->atracks[0]), trak, file->wr,
-                                         (lqt_codec_info_t*)0);
-		file->atracks[file->total_atracks].track = trak;
-		file->atracks[file->total_atracks].channels = channels;
-		file->atracks[file->total_atracks].current_position = 0;
-		file->atracks[file->total_atracks].current_chunk = 1;
-		file->total_atracks++;
-	}
-        lqt_set_default_audio_parameters(file);
-	return 1;   /* Return the number of tracks created */
+        lqt_codec_info_t ** info;
+        info = lqt_find_audio_codec(compressor, 1);
+        lqt_set_audio(file, channels, sample_rate, bits, *info);
+        lqt_destroy_codec_info(info);
+        return 1;   /* Return the number of tracks created */
 }
 
-int lqt_set_audio(quicktime_t *file, int channels,
-                  long sample_rate,  int bits,
-                  lqt_codec_info_t * codec_info)
-{
-         char * compressor = codec_info->fourccs[0];
-         quicktime_trak_t *trak;
+int lqt_set_video(quicktime_t *file, 
+                  int tracks, 
+                  int frame_w, 
+                  int frame_h,
+                  int frame_duration,
+                  int timescale,
+                  lqt_codec_info_t * info)
+  {
+	int i;
 
-/* allocate an arbitrary number of tracks */
-	if(channels)
-	{
-/* Fake the bits parameter for some formats. */
-		if(quicktime_match_32(compressor, QUICKTIME_ULAW) ||
-                   quicktime_match_32(compressor, QUICKTIME_IMA4))
-                  bits = 16;
-
-		file->atracks =
-                  (quicktime_audio_map_t*)calloc(1, sizeof(quicktime_audio_map_t));
-		trak = quicktime_add_track(file);
-		quicktime_trak_init_audio(file, trak, channels,
-                                          sample_rate, bits, compressor);
-
-                quicktime_init_audio_map(&(file->atracks[0]), trak, file->wr,
-                                         codec_info);
-		file->atracks[file->total_atracks].track = trak;
-		file->atracks[file->total_atracks].channels = channels;
-		file->atracks[file->total_atracks].current_position = 0;
-		file->atracks[file->total_atracks].current_chunk = 1;
-		file->total_atracks++;
-	}
-        lqt_set_default_audio_parameters(file);
-	return 1;   /* Return the number of tracks created */
+        for(i = 0; i < tracks; i++)
+          lqt_add_video_track(file, frame_w, frame_h, frame_duration, timescale, info);
+        return 0;
 }
 
-
 int quicktime_set_video(quicktime_t *file, 
                         int tracks, 
                         int frame_w, 
@@ -312,56 +432,38 @@ int quicktime_set_video(quicktime_t *file,
                         double frame_rate,
                         char *compressor)
   {
-	int i;
-	quicktime_trak_t *trak;
-
-	if(tracks)
-	{
-                quicktime_mhvd_init_video(file, &(file->moov.mvhd), frame_rate);
-		file->total_vtracks = tracks;
-		file->vtracks = (quicktime_video_map_t*)calloc(1, sizeof(quicktime_video_map_t) * file->total_vtracks);
-
-		for(i = 0; i < tracks; i++)
-		{
-			trak = quicktime_add_track(file);
-			quicktime_trak_init_video(file, trak, frame_w, frame_h, frame_rate, compressor);
-			quicktime_init_video_map(&(file->vtracks[i]), trak, file->wr, (lqt_codec_info_t*)0);
-		}
-//printf("quicktime_set_video 4\n");
-	}
-//printf("quicktime_set_video 5\n");
-        lqt_set_default_video_parameters(file);
-	return 0;
+        lqt_codec_info_t ** info;
+        int timescale, frame_duration;
+        timescale = quicktime_get_timescale(frame_rate);
+        frame_duration = (int)((double)(timescale)/frame_rate+0.5);
+        info = lqt_find_video_codec(compressor, 1);
+        lqt_set_video(file, tracks, frame_w, frame_h, frame_duration, timescale, *info);
+        lqt_destroy_codec_info(info);
+        return 0;
 }
 
 
-int lqt_set_video(quicktime_t *file, 
-                  int tracks, 
-                  int frame_w, 
-                  int frame_h,
-                  double frame_rate,
-                  lqt_codec_info_t * info)
+int lqt_add_video_track(quicktime_t *file,
+                        int frame_w, int frame_h,
+                        int frame_duration, int timescale,
+                        lqt_codec_info_t * info)
   {
-	int i;
         char * compressor = info->fourccs[0];
 	quicktime_trak_t *trak;
+        if(!file->total_vtracks)
+          quicktime_mhvd_init_video(file, &(file->moov.mvhd), timescale);
+        file->vtracks = realloc(file->vtracks, (file->total_vtracks+1) * sizeof(quicktime_video_map_t));
+        memset(&(file->vtracks[file->total_vtracks]), 0, sizeof(quicktime_video_map_t));
+        trak = quicktime_add_track(file);
+
+        quicktime_trak_init_video(file, trak, frame_w, frame_h, frame_duration, timescale, compressor);
+	quicktime_init_video_map(&(file->vtracks[file->total_vtracks]), trak, file->wr, info);
+        lqt_set_default_video_parameters(file, file->total_vtracks);
+        file->total_vtracks++;
+        return 0;
+  }
 
-	if(tracks)
-	{
-		quicktime_mhvd_init_video(file, &(file->moov.mvhd), frame_rate);
-		file->total_vtracks = tracks;
-		file->vtracks = (quicktime_video_map_t*)calloc(1, sizeof(quicktime_video_map_t) * file->total_vtracks);
 
-		for(i = 0; i < tracks; i++)
-		{
-			trak = quicktime_add_track(file);
-			quicktime_trak_init_video(file, trak, frame_w, frame_h, frame_rate, compressor);
-			quicktime_init_video_map(&(file->vtracks[i]), trak, file->wr, info);
-		}
-	}
-        lqt_set_default_video_parameters(file);
-	return 0;
-}
 
 
 void quicktime_set_framerate(quicktime_t *file, double framerate)
@@ -389,7 +491,7 @@ quicktime_trak_t* quicktime_add_track(quicktime_t *file)
         quicktime_moov_t *moov = &(file->moov);
         quicktime_trak_t *trak;
         int i;
-                                                                                                                  
+
         for(i = moov->total_tracks; i > 0; i--)
                 moov->trak[i] = moov->trak[i - 1];
                                                                                                                   
@@ -569,16 +671,19 @@ int quicktime_update_positions(quicktime_t *file)
 
 int quicktime_set_audio_position(quicktime_t *file, int64_t sample, int track)
 {
-	int64_t offset, chunk_sample, chunk;
-	quicktime_trak_t *trak;
 	if(track < file->total_atracks)
 	{
+#if 0 /* Old version (hopefully obsolete) */
 		trak = file->atracks[track].track;
                 file->atracks->current_position = sample;
                 quicktime_chunk_of_sample(&chunk_sample, &chunk, trak, sample);
 		file->atracks[track].current_chunk = chunk;
 		offset = quicktime_sample_to_offset(file, trak, sample);
 		quicktime_set_position(file, offset);
+#else
+                file->atracks->current_position = sample;
+                /* Codec will do the rest */
+#endif
 	}
 	else
 		fprintf(stderr, "quicktime_set_audio_position: track >= file->total_atracks\n");
@@ -694,16 +799,23 @@ int quicktime_has_video(quicktime_t *file)
 
 int quicktime_video_width(quicktime_t *file, int track)
 {
-	if(file->total_vtracks)
-		return file->vtracks[track].track->tkhd.track_width;
-	return 0;
+//	if(file->total_vtracks)
+//          return file->vtracks[track].track->tkhd.track_width;
+//	return 0;
+  if((track < 0) || (track >= file->total_vtracks))
+    return 0;
+  return file->vtracks[track].track->mdia.minf.stbl.stsd.table->width;
+  
 }
 
 int quicktime_video_height(quicktime_t *file, int track)
 {
-	if(file->total_vtracks)
-		return file->vtracks[track].track->tkhd.track_height;
-	return 0;
+//	if(file->total_vtracks)
+//          return file->vtracks[track].track->tkhd.track_height;
+//	return 0;
+  if((track < 0) || (track >= file->total_vtracks))
+    return 0;
+  return file->vtracks[track].track->mdia.minf.stbl.stsd.table->height;
 }
 
 int quicktime_video_depth(quicktime_t *file, int track)
@@ -722,7 +834,10 @@ void quicktime_set_cmodel(quicktime_t *file, int colormodel)
 
 void lqt_set_cmodel(quicktime_t *file, int track, int colormodel)
   {
-  file->vtracks[track].color_model = colormodel;
+  if((track < file->total_vtracks) && (track >= 0))
+    file->vtracks[track].color_model = colormodel;
+  else
+    fprintf(stderr, "lqt_set_cmodel: No track No. %d\n", track);
   }
 
 void quicktime_set_row_span(quicktime_t *file, int row_span)
@@ -737,6 +852,11 @@ void lqt_set_row_span(quicktime_t *file, int track, int row_span)
   file->vtracks[track].row_span = row_span;
   }
 
+void lqt_set_row_span_uv(quicktime_t *file, int track, int row_span_uv)
+  {
+  file->vtracks[track].row_span_uv = row_span_uv;
+  }
+
 void quicktime_set_window(quicktime_t *file,
         int in_x,                    /* Location of input frame to take picture */
         int in_y,
@@ -881,7 +1001,6 @@ int quicktime_write_audio(quicktime_t *file,
 
 int quicktime_write_frame(quicktime_t *file, unsigned char *video_buffer, int64_t bytes, int track)
 {
-        int64_t offset = quicktime_position(file);
         int result = 0;
         quicktime_atom_t chunk_atom;
         quicktime_video_map_t *vtrack = &file->vtracks[track];
@@ -899,16 +1018,16 @@ int quicktime_write_frame(quicktime_t *file, unsigned char *video_buffer, int64_
         return result;
 }
 
-
+#if 0 /* This function is obsolete as it makes no sense for compressed codecs */
 long quicktime_read_audio(quicktime_t *file, char *audio_buffer, long samples, int track)
 {
 	int64_t chunk_sample, chunk;
-	int result = 0, track_num;
+	int result = 0;
 	quicktime_trak_t *trak = file->atracks[track].track;
 	int64_t fragment_len, chunk_end;
 	int64_t start_position = file->atracks[track].current_position;
 	int64_t position = file->atracks[track].current_position;
-	int64_t start = position, end = position + samples;
+	int64_t end = position + samples;
 	int64_t bytes, total_bytes = 0;
 	int64_t buffer_offset;
 
@@ -939,6 +1058,7 @@ long quicktime_read_audio(quicktime_t *file, char *audio_buffer, long samples, i
 	if(result) return 0;
 	return total_bytes;
 }
+#endif
 
 int quicktime_read_chunk(quicktime_t *file, char *output, int track, int64_t chunk, int64_t byte_start, int64_t byte_len)
 {
@@ -976,14 +1096,13 @@ long quicktime_read_frame(quicktime_t *file, unsigned char *video_buffer, int tr
 	int64_t bytes;
 	int result = 0;
 
-	quicktime_trak_t *trak = file->vtracks[track].track;
 	bytes = quicktime_frame_size(file, file->vtracks[track].current_position, track);
 
 	quicktime_set_video_position(file, file->vtracks[track].current_position, track);
 	result = quicktime_read_data(file, video_buffer, bytes);
-	file->vtracks[track].current_position++;
+        lqt_update_frame_position(&file->vtracks[track]);
 
-	if(!result) return 0;
+        if(!result) return 0;
 	return bytes;
 }
 
@@ -1102,7 +1221,6 @@ int quicktime_has_keyframes(quicktime_t *file, int track)
 
 int quicktime_read_frame_init(quicktime_t *file, int track)
 {
-	quicktime_trak_t *trak = file->vtracks[track].track;
 	quicktime_set_video_position(file, file->vtracks[track].current_position, track);
 	if(quicktime_ftell(file) != file->file_position) 
 	{
@@ -1134,7 +1252,6 @@ int quicktime_init_video_map(quicktime_video_map_t *vtrack,
 
 int quicktime_delete_video_map(quicktime_video_map_t *vtrack)
 {
-	int i;
 	quicktime_delete_vcodec(vtrack);
 	return 0;
 }
@@ -1151,9 +1268,8 @@ int quicktime_init_audio_map(quicktime_audio_map_t *atrack, quicktime_trak_t *tr
 
 int quicktime_delete_audio_map(quicktime_audio_map_t *atrack)
 {
-	int i;
 	quicktime_delete_acodec(atrack);
-	return 0;
+        return 0;
 }
 
 void quicktime_init_maps(quicktime_t * file)
@@ -1161,38 +1277,43 @@ void quicktime_init_maps(quicktime_t * file)
   int i, track;
   /* get tables for all the different tracks */
   file->total_atracks = quicktime_audio_tracks(file);
-  file->atracks = (quicktime_audio_map_t*)calloc(1, sizeof(quicktime_audio_map_t) * file->total_atracks);
-  
-  for(i = 0, track = 0; i < file->total_atracks; i++)
+
+  if(file->total_atracks)
     {
-    while(!file->moov.trak[track]->mdia.minf.is_audio)
-      track++;
-    quicktime_init_audio_map(&(file->atracks[i]), file->moov.trak[track],
-                             file->wr,
-                             (lqt_codec_info_t*)0);
+    file->atracks = (quicktime_audio_map_t*)calloc(1, sizeof(quicktime_audio_map_t) * file->total_atracks);
+    for(i = 0, track = 0; i < file->total_atracks; i++)
+      {
+      while(!file->moov.trak[track]->mdia.minf.is_audio)
+        track++;
+      quicktime_init_audio_map(&(file->atracks[i]), file->moov.trak[track],
+                               file->wr,
+                               (lqt_codec_info_t*)0);
+      }
     }
-  
+
   file->total_vtracks = quicktime_video_tracks(file);
-  file->vtracks = (quicktime_video_map_t*)calloc(1, sizeof(quicktime_video_map_t) * file->total_vtracks);
-  
-  for(track = 0, i = 0; i < file->total_vtracks; i++)
+
+  if(file->total_vtracks)
     {
-    while(!file->moov.trak[track]->mdia.minf.is_video)
-      track++;
+    file->vtracks = (quicktime_video_map_t*)calloc(1, sizeof(quicktime_video_map_t) * file->total_vtracks);
     
-    quicktime_init_video_map(&(file->vtracks[i]), file->moov.trak[track],
-                             file->wr,
-                             (lqt_codec_info_t*)0);
+    for(track = 0, i = 0; i < file->total_vtracks; i++)
+      {
+      while(!file->moov.trak[track]->mdia.minf.is_video)
+        track++;
+      
+      quicktime_init_video_map(&(file->vtracks[i]), file->moov.trak[track],
+                               file->wr,
+                               (lqt_codec_info_t*)0);
+      }
     }
   }
 
 int quicktime_read_info(quicktime_t *file)
 {
         int result = 0, got_header = 0;
-        int i, channel, trak_channel, track;
         int64_t start_position = quicktime_position(file);
         quicktime_atom_t leaf_atom;
-        quicktime_trak_t *trak;
         char avi_avi[4];
         int got_avi = 0;
         int got_asf = 0;
@@ -1252,8 +1373,9 @@ int quicktime_read_info(quicktime_t *file)
                                                                                                                   
 //printf("quicktime_read_info 10\n");
 /* Construct indexes. */
-                quicktime_import_avi(file);
-//printf("quicktime_read_info 20\n");
+                if(quicktime_import_avi(file))
+                  return 1;
+                //printf("quicktime_read_info 20\n");
         }
 /* Quicktime section */
         else
@@ -1400,8 +1522,8 @@ void quicktime_set_asf(quicktime_t *file, int value)
 
 quicktime_t* quicktime_open(char *filename, int rd, int wr)
 {
+        int i;
 	quicktime_t *new_file = calloc(1, sizeof(quicktime_t));
-	char flags[10];
 	int result = 0;
         if(rd && wr)
           {
@@ -1463,8 +1585,11 @@ quicktime_t* quicktime_open(char *filename, int rd, int wr)
 	if(rd && new_file)
           {
 		/* Set default decoding parameters */
-          lqt_set_default_audio_parameters(new_file);
-          lqt_set_default_video_parameters(new_file);
+          for(i = 0; i < new_file->total_atracks; i++)
+            lqt_set_default_audio_parameters(new_file, i);
+
+          for(i = 0; i < new_file->total_vtracks; i++)
+            lqt_set_default_video_parameters(new_file, i);
           }
         
 	return new_file;
@@ -1491,8 +1616,9 @@ int quicktime_close(quicktime_t *file)
                         quicktime_finalize_odml(file, &file->riff[0]->hdrl);
                                                                                                                   
 // Finalize super indexes
-                        quicktime_finalize_indx(file);
-                                                                                                                  
+                        if(file->total_riffs > 1)
+                          quicktime_finalize_indx(file);
+                        
 // Pad ending
                         quicktime_set_position(file, position);
                         quicktime_atom_write_header(file, &junk_atom, "JUNK");
@@ -1530,6 +1656,8 @@ static void apply_default_parameters(quicktime_t * file,
   int num_parameters;
   lqt_parameter_info_t * parameter_info;
   int j;
+
+  //  fprintf(stderr, "APPLY CODEC INFO %s %d\n", codec_info->name, encode);
   
   if(encode)
     {
@@ -1572,24 +1700,73 @@ static void apply_default_parameters(quicktime_t * file,
     }                      
   }
 
-void lqt_set_default_video_parameters(quicktime_t * file)
+void lqt_set_default_video_parameters(quicktime_t * file, int track)
   {
-  int i;
   lqt_codec_info_t ** codec_info;
   quicktime_codec_t * codec;
-  for(i = 0; i < file->total_vtracks; i++)
+    
+  codec = (quicktime_codec_t*)(file->vtracks[track].codec);
+  codec_info = lqt_find_video_codec_by_name(codec->codec_name);
+  if(codec_info)
     {
-    codec = (quicktime_codec_t*)(file->vtracks[i].codec);
-    codec_info = lqt_find_video_codec_by_name(codec->codec_name);
-    if(codec_info)
-      {
-      apply_default_parameters(file, i, codec, *codec_info, file->wr);
-      lqt_destroy_codec_info(codec_info);
-      }
+    apply_default_parameters(file, track, codec, *codec_info, file->wr);
+    lqt_destroy_codec_info(codec_info);
     }
   }
 
-void lqt_set_default_audio_parameters(quicktime_t * file)
+int lqt_set_fiel(quicktime_t *file, int track, int nfields, int dominance)
+	{
+	quicktime_stsd_table_t *stsdt_p;
+
+	if	((track < 0) || (track >= file->total_vtracks))
+		return 0;
+
+	if	(nfields !=1 && nfields != 2)
+		return 0;
+
+/*
+ * http://developer.apple.com/quicktime/icefloe/dispatch019.html#fiel
+ *
+ * "dominance" is what Apple calls "detail".  From what I can figure out
+ * the term "bottom field first" corresponds to a "detail" setting of 14 and
+ * "top field first" is a "detail" setting of 9.
+*/
+	switch	(dominance)
+		{
+		case	0:
+		case	1:
+		case	6:
+		case	9:
+		case	14:
+			break;
+		default:
+			return 0;
+		}
+
+	stsdt_p = file->vtracks[track].track->mdia.minf.stbl.stsd.table;
+	stsdt_p->fields = nfields;
+	stsdt_p->field_dominance = dominance;
+	return 1;
+	}
+	
+int lqt_get_fiel(quicktime_t *file, int track, int *nfields, int *dominance)
+	{
+	quicktime_stsd_table_t *stsdt_p;
+
+	if	((track < 0) || (track >= file->total_vtracks))
+		return 0;
+
+	stsdt_p = file->vtracks[track].track->mdia.minf.stbl.stsd.table;
+
+	if	(nfields != NULL)
+		*nfields = stsdt_p->fields;
+	
+	if	(dominance != NULL)
+		*dominance = stsdt_p->field_dominance;
+	return 1;
+	}
+
+void lqt_set_default_audio_parameters(quicktime_t * file, int track)
   {
   int i;
   lqt_codec_info_t ** codec_info;
@@ -1718,3 +1895,87 @@ int64_t * lqt_get_chunk_sizes(quicktime_t * file, quicktime_trak_t *trak)
   free(chunk_indices);
   return ret;
   }
+
+int lqt_read_audio_chunk(quicktime_t * file, int track,
+                         long chunk,
+                         uint8_t ** buffer, int * buffer_alloc)
+  {
+  int64_t offset;
+  quicktime_trak_t * trak;
+  int result;
+
+  trak = file->atracks[track].track;
+
+  if(chunk >= file->atracks[track].track->mdia.minf.stbl.stco.total_entries)
+    {
+    return 0;
+    }
+  
+  if(!trak->chunk_sizes)
+    {
+    trak->chunk_sizes = lqt_get_chunk_sizes(file, trak);
+    }
+
+  /* Reallocate buffer */
+
+  if(*buffer_alloc < trak->chunk_sizes[chunk-1] + 16)
+    {
+    *buffer_alloc = trak->chunk_sizes[chunk-1] + 32;
+    *buffer = realloc(*buffer, *buffer_alloc);
+    }
+  
+  /* Get offset */
+  
+  offset = quicktime_chunk_to_offset(file, trak, chunk);
+
+  quicktime_set_position(file, offset);
+
+  result = quicktime_read_data(file, *buffer, trak->chunk_sizes[chunk-1]);
+
+  memset((*buffer) + trak->chunk_sizes[chunk-1], 0, 16);
+  
+  return result ? trak->chunk_sizes[chunk-1] : 0;
+  }
+
+int lqt_append_audio_chunk(quicktime_t * file, int track,
+                           long chunk,
+                           uint8_t ** buffer, int * buffer_alloc,
+                           int initial_bytes)
+  {
+  int64_t offset;
+  quicktime_trak_t * trak;
+  int result;
+
+  trak = file->atracks[track].track;
+
+  if(!trak->chunk_sizes)
+    {
+    trak->chunk_sizes = lqt_get_chunk_sizes(file, trak);
+    }
+
+  /* Reallocate buffer */
+
+  if(*buffer_alloc < trak->chunk_sizes[chunk-1] + 16 + initial_bytes)
+    {
+    *buffer_alloc = trak->chunk_sizes[chunk-1] + 32 + initial_bytes;
+    *buffer = realloc(*buffer, *buffer_alloc);
+    }
+  
+  /* Get offset */
+  
+  offset = quicktime_chunk_to_offset(file, trak, chunk);
+
+  quicktime_set_position(file, offset);
+
+  result = quicktime_read_data(file, (*buffer) + initial_bytes, trak->chunk_sizes[chunk-1]);
+
+  memset((*buffer) + initial_bytes + trak->chunk_sizes[chunk-1], 0, 16);
+  
+  return result ? trak->chunk_sizes[chunk-1] : 0;
+  }
+
+
+int64_t lqt_last_audio_position(quicktime_t * file, int track)
+  {
+  return file->atracks[track].last_position;
+  }
