@@ -1,10 +1,12 @@
 /*
- * simple quicktime movie player, needs libquicktime (or quicktime4linux).
+ * simple quicktime movie player, needs libquicktime
  *
  *  (c) 2002 Gerd Knorr <kraxel@bytesex.org>
  *
  */
 
+#define USE_GL
+
 #include "config.h"
 
 #include <stdio.h>
@@ -22,6 +24,9 @@
 #ifdef	HAVE_SOUNDCARD_H
 #include <sys/soundcard.h>
 #endif
+#ifdef HAVE_ALSA
+#include <alsa/asoundlib.h>
+#endif
 
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
@@ -32,13 +37,21 @@
 #include <X11/extensions/Xv.h>
 #include <X11/extensions/Xvlib.h>
 
+#ifdef USE_GL
+
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <GL/glx.h>
+#endif
 
 #include <quicktime/lqt.h>
 #include <quicktime/colormodels.h>
 
+/* File handle */
+
+static quicktime_t *qt = NULL;
+
+
 /* ------------------------------------------------------------------------ */
 /* X11 code                                                                 */
 
@@ -60,12 +73,29 @@ static int xv_have_YV12 = 0;
 static int no_mitshm    = 0;
 static int pixmap_bytes = 0;
 static int x11_byteswap = 0;
+
+#ifdef USE_GL
 static int use_gl       = 0;
+#endif
 
 static unsigned long   lut_red[256];
 static unsigned long   lut_green[256];
 static unsigned long   lut_blue[256];
 
+/* Video stuff */
+
+static int qt_cmodel = BC_RGB888;
+
+static int qt_cmodels[] =
+  {
+    BC_RGB888, /* Always supported */
+    /* Placeholders for various YUV formats, set by xv_init */
+    LQT_COLORMODEL_NONE,
+    LQT_COLORMODEL_NONE,
+    LQT_COLORMODEL_NONE,
+  };
+
+
 #define SWAP2(x) (((x>>8) & 0x00ff) |\
                   ((x<<8) & 0xff00))
 
@@ -192,6 +222,7 @@ static void x11_init(void)
 
 static void xv_init(void)
 {
+    int cmodel_index;
     int ver, rel, req, ev, err, i;
     int adaptors, formats;
     XvAdaptorInfo        *ai;
@@ -242,6 +273,14 @@ static void xv_init(void)
 	}
 	fprintf(stderr,"\n");
     }
+
+    /* Fill the cmodel array */
+    cmodel_index = 1;
+    if(xv_have_YUY2)
+      qt_cmodels[cmodel_index++] = BC_YUV422;
+    if(xv_have_YV12 || xv_have_I420)
+      qt_cmodels[cmodel_index++] = BC_YUV420P;
+    
 }
 
 static int
@@ -397,13 +436,14 @@ static void xv_blit(Window win, GC gc, XvImage *xi,
 
 /* ------------------------------------------------------------------------ */
 /* OpenGL code                                                              */
-
-static int tw,th;
-static GLint tex;
+#ifdef USE_GL
+static int gl_texture_width,gl_texture_height;
+static GLint gl_texture;
 static int gl_attrib[] = { GLX_RGBA,
-			   GLX_RED_SIZE, 1,
-			   GLX_GREEN_SIZE, 1,
-			   GLX_BLUE_SIZE, 1,
+                           GLX_RED_SIZE, 8,
+                           GLX_GREEN_SIZE, 8,
+                           GLX_BLUE_SIZE, 8,
+                           GLX_DEPTH_SIZE, 8,
 			   GLX_DOUBLEBUFFER,
 			   None };
 
@@ -424,21 +464,22 @@ static void gl_blit(Widget widget, char *rgbbuf,
     char *dummy;
     float x,y;
 
-    if (0 == tex) {
-	glGenTextures(1,&tex);
-	glBindTexture(GL_TEXTURE_2D,tex);
+    if (0 == gl_texture) {
+	glGenTextures(1,&gl_texture);
+	glBindTexture(GL_TEXTURE_2D,gl_texture);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-	dummy = malloc(tw*th*3);
-	memset(dummy,128,tw*th*3);
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,tw,th,0,
+	dummy = malloc(gl_texture_width*gl_texture_height*3);
+	memset(dummy,128,gl_texture_width*gl_texture_height*3);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,gl_texture_width,
+                     gl_texture_height,0,
 		     GL_RGB,GL_UNSIGNED_BYTE,dummy);
 	free(dummy);
     }
     glTexSubImage2D(GL_TEXTURE_2D, 0, 0,0,iw,ih,
 		    GL_RGB,GL_UNSIGNED_BYTE,rgbbuf);
-    x = (float)iw/tw;
-    y = (float)ih/th;
+    x = (float)iw/gl_texture_width;
+    y = (float)ih/gl_texture_height;
 
     glEnable(GL_TEXTURE_2D);
     glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
@@ -469,25 +510,30 @@ static void gl_init(Widget widget, int iw, int ih)
     glXMakeCurrent(XtDisplay(widget),XtWindow(widget),ctx);
     fprintf(stderr, "INFO: gl: DRI=%s\n",
 	    glXIsDirect(dpy, ctx) ? "Yes" : "No");
-    if (!glXIsDirect(dpy, ctx))
-	return;
-    
+    if (!glXIsDirect(dpy, ctx)) {
+        fprintf(stderr, "WARNING: gl: Direct rendering missing\n");
+        return;
+    }
+#if 0
     /* check against max size */
     glGetIntegerv(GL_MAX_TEXTURE_SIZE,&i);
     fprintf(stderr,"INFO: gl: texture max size: %d\n",i);
-    if (iw > i)
-	return;
-    if (ih > i)
-	return;
-    
+    if ((iw > i) || (ih > i))
+        {
+        fprintf(stderr, "WARNING: gl: Maximum texture size too small (got %dx%d, needed %dx%d)\n",
+                i, i, iw, ih);
+        return;
+        }
+#endif
     /* textures have power-of-two x,y dimensions */
     for (i = 0; iw >= (1 << i); i++)
 	;
-    tw = (1 << i);
+    gl_texture_width = (1 << i);
     for (i = 0; ih >= (1 << i); i++)
 	;
-    th = (1 << i);
-    fprintf(stderr,"INFO: gl: frame=%dx%d, texture=%dx%d\n",iw,ih,tw,th);
+    gl_texture_height = (1 << i);
+    fprintf(stderr,"INFO: gl: frame=%dx%d, texture=%dx%d\n",iw,ih,
+            gl_texture_width,gl_texture_height);
 
     glClearColor (0.0, 0.0, 0.0, 0.0);
     glShadeModel(GL_FLAT);
@@ -495,6 +541,146 @@ static void gl_init(Widget widget, int iw, int ih)
     use_gl = 1;
 }
 
+#endif // USE_GL
+
+static int oss_sr,oss_hr;
+
+
+
+/* ------------------------------------------------------------------------ */
+/* alsa code                                                                 */
+
+#ifdef HAVE_ALSA
+
+/* Enable alsa */
+static int use_alsa = 1;
+
+#ifndef SND_PCM_FORMAT_S16_NE
+# ifdef WORDS_BIGENDIAN
+#  define SND_PCM_FORMAT_S16_NE SND_PCM_FORMAT_S16_BE
+# else
+#  define SND_PCM_FORMAT_S16_NE SND_PCM_FORMAT_S16_LE
+# endif
+#endif
+
+//static int periods = 32; /* number of periods == fragments */
+static snd_pcm_uframes_t periodsize = 1024; /* Periodsize (bytes) */
+static unsigned int buffer_time = 500000; 
+static unsigned int period_time = 125000;              /* period time in us */
+
+/* Handle for the PCM device */ 
+static snd_pcm_t *pcm_handle;
+
+/* This structure contains information about    */
+/* the hardware and can be used to specify the  */
+/* configuration to be used for the PCM stream. */
+static snd_pcm_hw_params_t *hwparams;
+snd_pcm_sframes_t buffer_size;
+
+#else
+
+/* Disable Alsa */
+static int use_alsa = 0;
+
+#endif /* HAVE_ALSA */
+
+static int alsa_init(char *dev, int channels, int rate)
+{
+#ifdef HAVE_ALSA
+    int dir;
+    //    int exact_param;   /* parameter returned by          */
+                       /* snd_pcm_hw_params_set_*_near   */ 
+    int tmprate;
+    int err = 0;
+    tmprate = rate;
+    oss_hr = rate;
+    if (snd_pcm_open(&pcm_handle, dev, SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK) < 0) {
+	fprintf(stderr, "Error opening PCM device %s\n", dev);
+	return 1;
+    }
+    
+    /* Allocate the snd_pcm_hw_params_t structure on the stack. */
+    snd_pcm_hw_params_alloca(&hwparams);
+
+    if ((err = snd_pcm_hw_params_any(pcm_handle, hwparams)) < 0) {
+	fprintf(stderr, "Can not configure this PCM device. (%s)\n", snd_strerror(err));
+	return 1;
+    }
+    
+    if ((err = snd_pcm_hw_params_set_access(pcm_handle, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
+	fprintf(stderr, "Error setting access. (%s)\n", snd_strerror(err));
+	return 1;
+    }
+
+    /* put checks here */
+    /* Only needed when sampling format not supported by hardware .. unlikely */
+
+    /* Set sample format */
+    if ((err = snd_pcm_hw_params_set_format(pcm_handle, hwparams,  SND_PCM_FORMAT_S16_NE)) < 0) {
+	fprintf(stderr, "Error setting format.(%s)\n", snd_strerror(err));
+	return 1;
+    }
+    
+    /* Set number of channels */ // weird 1 channel mode doesn't work
+    if ((err = snd_pcm_hw_params_set_channels(pcm_handle, hwparams, channels)) < 0) {
+	fprintf(stderr, "Error setting channels. %i (%s)\n", channels, snd_strerror(err));
+	return 1;
+    }
+    
+    /* Set sample rate. If the exact rate is not supported */
+    /* by the hardware, use nearest possible rate.         */ 
+    if ((err = snd_pcm_hw_params_set_rate_near(pcm_handle, hwparams, &tmprate, 0)) < 0 ) {
+	fprintf(stderr, "Error setting sample rate (%s)\n", snd_strerror(err));
+	return 1;
+    }
+
+    if (tmprate != rate) fprintf(stderr,"WARNING: Using %i Hz instead of requested rate %i Hz\n ", tmprate, rate);
+    oss_sr = tmprate;
+    dir = 0;
+    if ((err = snd_pcm_hw_params_set_buffer_time_near(pcm_handle, hwparams, &buffer_time, &dir)) < 0) {
+	printf("Unable to set buffer time %i for playback: %s\n", buffer_time, snd_strerror(err));
+	return 1;
+    }
+
+    /* period time */
+    dir = 0;
+    if ((err = snd_pcm_hw_params_set_period_time_near(pcm_handle, hwparams, &period_time, &dir)) < 0) {
+	fprintf(stderr, "Error setting periods.(%s)\n", snd_strerror(err));
+	return 1;
+    }
+
+    
+    if ((err = snd_pcm_hw_params_get_buffer_size(hwparams, &buffer_size))< 0) {
+                fprintf(stderr, "Unable to get buffer size for playback: %s\n", snd_strerror(err));
+                return 1;
+    }
+    
+    dir = 0;
+    err = snd_pcm_hw_params_get_period_size(hwparams, &periodsize, &dir);
+    if (err < 0) {
+                fprintf(stderr, "Unable to get period size for playback: %s\n", snd_strerror(err));
+                return 1;
+    }
+
+    /* Apply HW parameter settings to */
+    /* PCM device and prepare device  */
+    if ((err = snd_pcm_hw_params(pcm_handle, hwparams)) < 0) {
+	fprintf(stderr, "Error setting HW params.(%s)\n", snd_strerror(err));
+	return 1;
+    }
+    
+    if ((err = snd_pcm_prepare(pcm_handle)) < 0) {
+	fprintf(stderr, "Error in pcm_prepare.(%s)\n", snd_strerror(err));
+	return 1;
+    }
+    
+#endif
+
+return 0;
+}
+
+
+
 /* ------------------------------------------------------------------------ */
 /* oss code                                                                 */
 
@@ -542,7 +728,7 @@ oss_setformat(int chan, int rate)
 
 static int oss_init(char *dev, int channels, int rate)
 {
-    int trigger;
+//    int trigger;
 
     oss_fd = open(dev,O_WRONLY | O_NONBLOCK);
     if (-1 == oss_fd) {
@@ -562,24 +748,48 @@ static int oss_init(char *dev, int channels, int rate)
 /* ------------------------------------------------------------------------ */
 /* quicktime code                                                           */
 
-static quicktime_t *qt = NULL;
+/* Audio stuff */
+
+/* Interleaved audio buffer */
+static int16_t *qt_audio = (int16_t*)0;    //,*qt1,*qt2;
+static int16_t *qt_audio_ptr; /* Pointer to the sample buffer for the next write() call */
+
+static int qt_audio_samples_in_buffer;
+
+/* Non interleaved audio buffer */
+static int16_t **qt_audion = (int16_t**)0;
+
+/* One decode call will decode this many samples */
+
+#define AUDIO_BLOCK_SIZE (10*1024)
+
+static int qt_channels,qt_sample_rate;
+static int qt_audio_eof = 0; /* No more samples can be decoded */
+
 static int qt_hasvideo,qt_hasaudio;
 
 static int qt_width = 320, qt_height = 32, qt_drop = 0, qt_droptotal = 0;
+static int64_t qt_frame_time; /* Timestamp of the decoded frame */
+static int qt_timescale = 0;
+
 static unsigned char *qt_frame,**qt_rows;
 static XImage *qt_ximage;
 static XvImage *qt_xvimage;
 static GC qt_gc;
 
-static int16_t *qt_audio,*qt1,*qt2;
-static int qt_size,qt_offset,qt_stereo;
-static int qt_cmodel = BC_RGB888;
 
 static void qt_init(FILE *fp, char *filename)
 {
     char *str;
     int i;
 
+    /* audio device */
+    char *adev_name;
+    
+    /* default */
+    adev_name = strdup("plughw");
+//    if (getenv("LQTPLAY_ALSA_DEV")) adev_name = getenv("LQTPLAY_ALSA_DEV");
+
     /* open file */
     qt = quicktime_open(filename,1,0);
     if (NULL == qt) {
@@ -594,10 +804,34 @@ static void qt_init(FILE *fp, char *filename)
 	fprintf(fp,"  copyright: %s\n",str);
     str = quicktime_get_name(qt);
     if (str)
-	fprintf(fp,"  name: %s\n",str);
+	fprintf(fp,"  name:      %s\n",str);
     str = quicktime_get_info(qt);
     if (str)
-	fprintf(fp,"  info: %s\n",str);
+	fprintf(fp,"  info:      %s\n",str);
+
+    str = lqt_get_author(qt);
+    if (str)
+	fprintf(fp,"  author:    %s\n",str);
+
+    str = lqt_get_artist(qt);
+    if (str)
+	fprintf(fp,"  artist:    %s\n",str);
+
+    str = lqt_get_album(qt);
+    if (str)
+        fprintf(fp,"  album:     %s\n",str);
+
+    str = lqt_get_genre(qt);
+    if (str)
+	fprintf(fp,"  genre:     %s\n",str);
+
+    str = lqt_get_track(qt);
+    if (str)
+	fprintf(fp,"  track:     %s\n",str);
+
+    str = lqt_get_comment(qt);
+    if (str)
+	fprintf(fp,"  comment:   %s\n",str);
     
     /* print video info */
     if (quicktime_has_video(qt)) {
@@ -646,6 +880,8 @@ static void qt_init(FILE *fp, char *filename)
 	qt_hasvideo = 1;
 	qt_width  = quicktime_video_width(qt,0);
 	qt_height = quicktime_video_height(qt,0);
+        qt_timescale = lqt_video_time_scale(qt,0);
+        fprintf(stderr, "Timescale: %d\n", qt_timescale);
     }
 
     if (!quicktime_has_audio(qt)) {
@@ -654,13 +890,19 @@ static void qt_init(FILE *fp, char *filename)
 	fprintf(stderr,"WARNING: unsupported audio codec\n");
     } else {
 	qt_hasaudio = 1;
-	if (quicktime_track_channels(qt,0) > 1)
-	    qt_stereo = 1;
-	if (-1 == oss_init("/dev/dsp", qt_stereo ? 2 : 1,
-			   quicktime_sample_rate(qt,0)))
-	    qt_hasaudio = 0;
-    }
-
+	qt_channels = quicktime_track_channels(qt,0);
+        qt_sample_rate = quicktime_sample_rate(qt,0);
+  if (use_alsa == 1) {
+    if (-1 == alsa_init(adev_name, qt_channels, 
+         qt_sample_rate)) {
+           qt_hasaudio = 0;}
+  }
+  else {
+  	if (-1 == oss_init("/dev/dsp", qt_channels,
+                           qt_sample_rate))
+           qt_hasaudio = 0;
+  }
+  }
     if (0 == qt_hasvideo && 0 == qt_hasaudio) {
 	fprintf(stderr,"ERROR: no playable stream found\n");
 	exit(1);
@@ -673,19 +915,22 @@ static void qt_cleanup()
     quicktime_close(qt);
   }
 
-static int qt_frame_blit(void)
+static int qt_init_video(void)
 {
-    int i;
-    
-    if (0 == quicktime_video_position(qt,0)) {
-	/* init */
+        int i;
+        /* init */
 	qt_frame = malloc(qt_width * qt_height * 4);
 	qt_rows = malloc(qt_height * sizeof(char*));
 	qt_gc = XCreateGC(dpy,XtWindow(simple),0,NULL);
 	switch (qt_cmodel) {
 	case BC_RGB888:
-	    fprintf(stderr,"INFO: using BC_RGB888 + %s\n",
+#ifdef USE_GL
+          fprintf(stderr,"INFO: using BC_RGB888 + %s\n",
 		    use_gl ? "OpenGL" : "plain X11");
+#else
+          fprintf(stderr,"INFO: using BC_RGB888 + %s\n",
+		    "plain X11");
+#endif
 	    qt_ximage = x11_create_ximage(dpy,qt_width,qt_height);
 	    for (i = 0; i < qt_height; i++)
 		qt_rows[i] = qt_frame + qt_width * 3 * i;
@@ -696,8 +941,11 @@ static int qt_frame_blit(void)
 					  xv_port,FOURCC_YUV2);
 	    for (i = 0; i < qt_height; i++)
 		qt_rows[i] = qt_xvimage->data + qt_width * 2 * i;
-	    break;
-	case BC_YUV420P:
+
+            lqt_set_row_span(qt,0,qt_xvimage->pitches[0]);
+            
+            break;
+        case BC_YUV420P:
             if(xv_have_YV12)
               {
               fprintf(stderr,
@@ -718,29 +966,50 @@ static int qt_frame_blit(void)
 	      qt_rows[1] = qt_xvimage->data + qt_xvimage->offsets[1];
 	      qt_rows[2] = qt_xvimage->data + qt_xvimage->offsets[2];
 	      }
-              break;
+            lqt_set_row_span(qt,0,qt_xvimage->pitches[0]);
+            lqt_set_row_span_uv(qt,0,qt_xvimage->pitches[1]);
+            break;
 	default:
 	    fprintf(stderr,"ERROR: internal error at %s:%d\n",
 		    __FILE__,__LINE__);
 	    exit(1);
 	}
-    }
+        return 0;
+}
 
-    if (qt_drop) {
-	qt_droptotal += qt_drop;
+static int qt_frame_decode(void)
+  {
+  int i;
+  if (quicktime_video_position(qt,0) >= quicktime_video_length(qt,0))
+      return -1;
+
+  if (qt_drop) {
+        qt_droptotal += qt_drop;
 	fprintf(stderr,"dropped %d frame(s)\r",qt_droptotal);
 	for (i = 0; i < qt_drop; i++)
 	    quicktime_read_frame(qt,qt_frame,0);
 	qt_drop = 0;
     }
+    qt_frame_time = lqt_frame_time(qt, 0);
+//    fprintf(stderr, "Frame time: %lld\n", qt_frame_time);
     lqt_decode_video(qt, qt_rows, 0);
 //    quicktime_decode_scaled(qt,0,0,qt_width,qt_height,qt_width,qt_height,
 //			    qt_cmodel,qt_rows,0);
+    return 0;
+  }
+
+static int qt_frame_blit(void)
+{
+
     switch (qt_cmodel) {
     case BC_RGB888:
-	if (use_gl) {
+#ifdef USE_GL
+      if (use_gl) {
 	    gl_blit(simple,qt_frame,qt_width,qt_height,swidth,sheight);
-	} else {
+	} else
+#endif // USE_GL
+
+        {
 	    switch (pixmap_bytes) {
 	    case 2:
 		rgb_to_lut2(qt_ximage->data,qt_frame,qt_width*qt_height);
@@ -763,8 +1032,6 @@ static int qt_frame_blit(void)
 	exit(1);
     }
 
-    if (quicktime_video_position(qt,0) >= quicktime_video_length(qt,0))
-	return -1;
     return 0;
 }
 
@@ -774,14 +1041,18 @@ static void qt_frame_delay(struct timeval *start, struct timeval *wait)
     long msec;
 
     gettimeofday(&now,NULL);
+    /* Get the now - start (->negative value) */
     msec  = (start->tv_sec  - now.tv_sec)  * 1000;
     msec += (start->tv_usec - now.tv_usec) / 1000;
     if (qt_hasaudio && oss_sr && oss_hr) {
 	/* cheap trick to make a/v sync ... */
 	msec = (long long)msec * oss_hr / oss_sr;
     }
-    msec += quicktime_video_position(qt,0) * 1000
-	/ quicktime_frame_rate(qt,0);
+    
+    msec += (qt_frame_time * 1000) / qt_timescale;
+
+    //    fprintf(stderr, "Time (msec) %ld\n", (qt_frame_time * 1000) / qt_timescale);
+
     if (msec < 0) {
 	qt_drop = -msec * quicktime_frame_rate(qt,0) / 1000;
 	wait->tv_sec  = 0;
@@ -792,39 +1063,112 @@ static void qt_frame_delay(struct timeval *start, struct timeval *wait)
     }
 }
 
-static int qt_audio_write(void)
+
+/* Decode at most AUDIO_BLOCK_SIZE samples and interleave them into the
+   qt_audio array. Return the real number of decoded samples */
+
+static int decode_audio()
+  {
+  int i, j;
+  int samples_decoded;
+  long last_pos = lqt_last_audio_position(qt, 0);
+  if(!qt_audio)
+    {
+    /* Initialize */
+
+    if(qt_channels > 1)
+      {
+      qt_audion  = malloc(qt_channels * sizeof(*qt_audion));
+      for(i = 0; i < qt_channels; i++)
+        {
+        qt_audion[i] = calloc(AUDIO_BLOCK_SIZE, sizeof(*(qt_audion[i])));
+        }
+      }
+    qt_audio  = calloc(AUDIO_BLOCK_SIZE*qt_channels, sizeof(*qt_audio));
+    }
+  
+  if(qt_channels > 1)
+    {
+    lqt_decode_audio_track(qt, qt_audion, (float**)0, AUDIO_BLOCK_SIZE, 0);
+    samples_decoded = lqt_last_audio_position(qt, 0) - last_pos;
+    
+    /* Interleave */
+    for (i = 0; i < samples_decoded; i++)
+      {
+      for(j = 0; j < qt_channels; j++)
+        {
+        qt_audio[qt_channels*i+j] = qt_audion[j][i];
+        }
+      }
+    }
+  else
+    {
+    //    fprintf(stderr, "Decode audio...");
+    lqt_decode_audio_track(qt, &qt_audio, (float**)0, AUDIO_BLOCK_SIZE, 0);
+    //    fprintf(stderr, "done\n");
+    samples_decoded = lqt_last_audio_position(qt, 0) - last_pos;
+    }
+  qt_audio_samples_in_buffer = samples_decoded;
+  qt_audio_ptr = qt_audio;
+  if(samples_decoded < AUDIO_BLOCK_SIZE)
+    {
+    fprintf(stderr, "Audio track finished\n");
+    qt_audio_eof = 1;
+    }
+  return samples_decoded;
+  }
+
+//static int runcount;
+static int qt_alsa_audio_write()
 {
-    long pos;
-    int rc,i;
-
-    if (0 == quicktime_audio_position(qt,0)) {
-	/* init */
-	qt_size   = 64 * 1024;
-	qt_offset = 0;
-	qt_audio  = malloc(qt_size);
-	if (quicktime_track_channels(qt,0) > 1) {
-	    qt1 = malloc(qt_size/2);
-	    qt2 = malloc(qt_size/2);
-	}
+#ifdef HAVE_ALSA
+    int done = 0;
+    int ret = 0;
+    //    fprintf(stderr, "Write alsa %d\n", qt_audio_samples_in_buffer);
+    while(!done) {
+        /* Decode new audio samples */
+        if(!qt_audio_samples_in_buffer)
+          decode_audio(AUDIO_BLOCK_SIZE);
+        //        fprintf(stderr, "snd_pcm_writei..");
+        ret = snd_pcm_writei(pcm_handle, (void *)(qt_audio_ptr), qt_audio_samples_in_buffer);
+        //        fprintf(stderr, "done %d\n", ret);
+        if (ret == -EAGAIN) { 
+            ret = 0;
+            done = 1;
+    //           snd_pcm_wait(pcm_handle, 1000);
+        }
+        else if (ret == -EPIPE) {
+            snd_pcm_prepare(pcm_handle);
+            fprintf(stderr, "Warning: buffer underrun\n");
+        }
+        else if (ret < 0) {
+            fprintf(stderr, "Warning: %s\n", snd_strerror(ret));
+        }
+        else if (ret >= 0)
+          done = 1;
     }
-
-    if (0 == qt_offset) {
-	if (qt_stereo) {
-	    /* stereo: two channels => interlaved samples */
-	    pos = quicktime_audio_position(qt,0);
-	    quicktime_decode_audio(qt,qt1,NULL,qt_size/4,0);
-	    quicktime_set_audio_position(qt,pos,0);
-	    quicktime_decode_audio(qt,qt2,NULL,qt_size/4,1);
-	    for (i = 0; i < qt_size/4; i++) {
-		qt_audio[2*i+0] = qt1[i];
-		qt_audio[2*i+1] = qt2[i];
-	    }
-	} else {
-	    /* mono */
-	    quicktime_decode_audio(qt,qt_audio,NULL,qt_size/2,0);
-	}
+    if (ret > 0 ) {
+    qt_audio_samples_in_buffer -=ret;
+    qt_audio_ptr += ret * qt_channels;
     }
-    rc = write(oss_fd,qt_audio+qt_offset/2,qt_size-qt_offset);
+    
+    if (qt_audio_eof && 0 == qt_audio_samples_in_buffer) {
+        snd_pcm_drain(pcm_handle);
+        return -1;
+    }
+#endif
+
+  return 0;
+}
+
+static int qt_oss_audio_write(void)
+{
+    int rc;
+
+    if(!qt_audio_samples_in_buffer)
+      decode_audio(AUDIO_BLOCK_SIZE);
+    
+    rc = write(oss_fd,qt_audio_ptr,qt_audio_samples_in_buffer * qt_channels * sizeof(*qt_audio));
     switch (rc) {
     case -1:
 	perror("write dsp");
@@ -839,14 +1183,14 @@ static int qt_audio_write(void)
 	qt_hasaudio = 0;
 	break;
     default:
-	qt_offset += rc;
-	if (qt_offset == qt_size)
-	    qt_offset = 0;
+        qt_audio_samples_in_buffer -= rc * qt_channels * sizeof(*qt_audio);
+        qt_audio_ptr += rc / sizeof(*qt_audio);
+        break;
     }
 
-    if (quicktime_audio_position(qt,0) >= quicktime_audio_length(qt,0) &&
-	0 == qt_offset)
-	return -1;
+    if (qt_audio_eof && 0 == qt_audio_samples_in_buffer) {
+       return -1;
+    }
     return 0;
 }
 
@@ -856,6 +1200,7 @@ static int qt_audio_write(void)
 struct ARGS {
     int  xv;
     int  gl;
+    int  alsa;
 } args;
 
 XtResource args_desc[] = {
@@ -871,6 +1216,11 @@ XtResource args_desc[] = {
 	XtCValue, XtRInt, sizeof(int),
 	XtOffset(struct ARGS*,gl),
 	XtRString, "1"
+    },{
+	"alsa",
+	XtCValue, XtRInt, sizeof(int),
+	XtOffset(struct ARGS*,alsa),
+	XtRString, "1"
     }
 };
 const int args_count = XtNumber(args_desc);
@@ -878,6 +1228,7 @@ const int args_count = XtNumber(args_desc);
 XrmOptionDescRec opt_desc[] = {
     { "-noxv",  "xv", XrmoptionNoArg,  "0" },
     { "-nogl",  "gl", XrmoptionNoArg,  "0" },
+    { "-noalsa",  "alsa", XrmoptionNoArg,  "0" },
 };
 const int opt_count = (sizeof(opt_desc)/sizeof(XrmOptionDescRec));
 
@@ -898,6 +1249,7 @@ static void usage(FILE *fp, char *prog)
 	    "options:\n"
 	    "  -noxv   don't use the Xvideo extention\n"
 	    "  -nogl   don't use OpenGL\n"
+	    "  -noalsa don't use Alsa\n"
 	    "\n",
 	    prog);
 }
@@ -916,8 +1268,10 @@ static void resize_ev(Widget widget, XtPointer client_data,
     case ConfigureNotify:
 	XtVaGetValues(widget,XtNheight,&sheight,XtNwidth,&swidth,NULL);
 	fprintf(stderr,"INFO: window size is %dx%d\n",swidth,sheight);
+#ifdef USE_GL
 	if (use_gl)
 	    gl_resize(widget,swidth,sheight);
+#endif
 	break;
     }
 }
@@ -936,7 +1290,9 @@ static String res[] = {
 
 int main(int argc, char *argv[])
 {
+    int has_frame = 0, blit_frame = 0;
     struct timeval start,wait;
+    //    int audio_frames;
     
     app_shell = XtVaAppInitialize(&app_context, "lqtplay",
 				  opt_desc, opt_count,
@@ -946,6 +1302,9 @@ int main(int argc, char *argv[])
 			      args_desc,args_count,
 			      NULL,0);
 
+    /* don't use alsa*/			      
+    if (!args.alsa) use_alsa=0;
+			      
     /* open file */
     if (argc < 2) {
 	usage(stderr,argv[0]);
@@ -963,24 +1322,34 @@ int main(int argc, char *argv[])
 				     XtNheight, qt_height,
 				     NULL);
     XtAddEventHandler(simple,StructureNotifyMask, True, resize_ev, NULL);
+
     x11_init();
     if (args.xv && qt_hasvideo)
 	xv_init();
-
-    /* use Xvideo? */
-    if (xv_have_YUY2 && quicktime_reads_cmodel(qt,BC_YUV422,0))
-	qt_cmodel = BC_YUV422;
-    if ((xv_have_I420 || xv_have_YV12) && quicktime_reads_cmodel(qt,BC_YUV420P,0))
-	qt_cmodel = BC_YUV420P;
-
-    /* Set decoding colormodel */
-    quicktime_set_cmodel(qt, qt_cmodel);
-
+    if(qt_hasvideo) {
+        /* Decide about the colormodel */
+        qt_cmodel = lqt_get_best_colormodel(qt, 0, qt_cmodels);
+        fprintf(stderr, "Using colormodel %s\n", lqt_colormodel_to_string(qt_cmodel));
+        /* Set decoding colormodel */
+        lqt_set_cmodel(qt, 0, qt_cmodel);
+    }
     /* use OpenGL? */
     XtRealizeWidget(app_shell);
+#ifdef USE_GL
     if (BC_RGB888 == qt_cmodel && args.gl && qt_hasvideo)
 	gl_init(simple,qt_width,qt_height);
-
+#endif    
+    /* frames per chunk for alsa */
+#if 0
+    if(qt_hasvideo)
+      audio_frames = ((oss_sr / quicktime_frame_rate(qt,0)) / 2 + 0.5);
+    else
+      audio_frames = oss_sr + 1024;
+#endif
+    /* Initialize video */
+    if(qt_hasvideo)
+      qt_init_video();
+    
     /* enter main loop */
     gettimeofday(&start,NULL);
     for (;;) {
@@ -997,25 +1366,58 @@ int main(int argc, char *argv[])
 	    FD_SET(ConnectionNumber(dpy),&rd);
 	    max = ConnectionNumber(dpy);
 	    if (qt_hasaudio) {
-		FD_SET(oss_fd,&wr);
-		if (oss_fd > max)
-		    max = oss_fd;
+		if (use_alsa == 0) {
+		    FD_SET(oss_fd,&wr);
+		    if (oss_fd > max)
+			max = oss_fd;
+		}
 	    }
 	    if (qt_hasvideo) {
-		qt_frame_delay(&start,&wait);
-	    } else {
-		wait.tv_sec  = 1;
-		wait.tv_usec = 0;
+                if(!has_frame) {
+                    if(0 != qt_frame_decode()) {
+                        qt_hasvideo = 0;
+                        wait.tv_sec  = 0;
+                        wait.tv_usec = 1000;
+                    } else {
+                    has_frame = 1;
+                    }
+                }
+                    
+            qt_frame_delay(&start,&wait);
+
+            /* "wait" is the time, we would have to wait.
+               If it's longer, than 2 ms, we'll continue feeding the
+               soundcard. This prevents audio underruns for frames with a
+               VERY long duration */
+             if(wait.tv_sec || (wait.tv_usec > 2000)) {
+                 wait.tv_sec  = 0;
+                 wait.tv_usec = 2000;
+                 blit_frame = 0;
+             } else
+                 blit_frame = 1;
+                
+	 } else {
+		wait.tv_sec  = 0;
+		wait.tv_usec = 1000;
 	    }
 	    rc = select(max+1,&rd,&wr,NULL,&wait);
-	    if (qt_hasaudio && FD_ISSET(oss_fd,&wr))
-		if (0 != qt_audio_write())
-		    break;
-	    if (qt_hasvideo && 0 == rc)
-		if (0 != qt_frame_blit())
-		    break;
+	    if (qt_hasaudio) {
+		if (use_alsa == 1) {
+		    if (0 != qt_alsa_audio_write()) qt_hasaudio = 0;
+		}
+		else if (FD_ISSET(oss_fd,&wr)) { 
+		    if (0 != qt_oss_audio_write()) qt_hasaudio = 0;
+		}
+	    }
+	    if (qt_hasvideo && 0 == rc && blit_frame) {
+		qt_frame_blit();
+                has_frame = 0;
+	    }
 	}
+    if(!qt_hasvideo && !qt_hasaudio)
+      break;
     }
-    qt_cleanup();    
+    qt_cleanup();
     return 0;
 }
+
